#include <stdio.h>
#include <stdarg.h>

#include "space.h"
#include "collide.h"

void Dump(char *s, ...)
{
  FILE *f;

  char temp[256];
  va_list args;

  va_start(args, s);
  vsprintf(temp, s, args);
  va_end(args);

  f = fopen("dump.txt", "a");
  fprintf(f, "%s", temp);
  fclose(f);
}

/*
    Collision detection routine.
*/
int DidCollide1(MBITMAP image1, MBITMAP image2, int x1, int y1,
               int x2, int y2)
{
  int xstart, ystart, xend, yend;
  unsigned char c1, c2;
  register int x, y;
  int collision = 0;

  xstart = x2 - x1;
  ystart = y2 - y1;
  xend = xstart + image2.x;
  if(xend > image1.x)
    xend = image1.x;
  yend = ystart + image2.y;
  if(yend > image1.y)
    yend = image1.y;

  if(xstart == xend || ystart == yend) return 0;

  y = ystart;
  do {
    x = xstart;
    do {
      c1 = image1.bits[(y * image1.x) + x];
      c2 = image2.bits[((y-ystart)*image2.x) + (x-xstart)];
      if(c1 != 255 && c2 != 255)
      {
        collision = 1;
        x = xend + 1;
        y = yend + 1;
      }
      x++;
    } while(x < xend);
    y++;
  } while(y < yend);

  return collision;
}

/*
    Collision detection routine.
*/
int DidCollide2(MBITMAP image1, MBITMAP image2, int x1, int y1,
               int x2, int y2)
{
  int xstart, ystart, xend, yend;
  unsigned char c1, c2;
  register int x, y;
  int collision = 0;

  xstart = x1 - x2;
  ystart = y1 - y2;
  xend = xstart + image1.x;
  if(xend > image2.x)
    xend = image2.x;
  yend = image1.y;
  if(yend > image2.y)
    yend = image2.y;

  if(xstart == xend || ystart == yend) return 0;

  y = ystart;
  do {
    x = xstart;
    do {
      c1 = image1.bits[((y-ystart)*image1.x) + (x-xstart)];
      c2 = image2.bits[(y * image2.x) + x];
      if(c1 != 255 && c2 != 255)
      {
        collision = 1;
        x = xend + 1;
        y = yend + 1;
      }
      x++;
    } while(x < xend);
    y++;
  } while(y < yend);

  return collision;
}


/*
    Collision detection routine.
*/
int DidCollide3(MBITMAP image1, MBITMAP image2, int x1, int y1,
               int x2, int y2)
{
  int xstart, ystart, xend, yend;
  unsigned char c1, c2;
  register int x, y;
  int collision = 0;

  xstart = x2 - x1;
  ystart = y1 - y2;
  xend = image1.x;
  if(xend > (x2 + image2.x))
    xend = x2 + image2.x;
  yend = image1.y;
  if(yend > image2.y)
    yend = image2.y;

  if(xstart == xend || ystart == yend) return 0;

  y = ystart;
  do {
    x = xstart;
    do {
      c1 = image1.bits[((y-ystart)*image1.x) + x];
      c2 = image2.bits[(y * image2.x) + (x - xstart)];
      if(c1 != 255 && c2 != 255)
      {
        collision = 1;
        x = xend + 1;
        y = yend + 1;
      }
      x++;
    } while(x < xend);
    y++;
  } while(y < yend);

  return collision;
}

/*
    Collision detection routine.
*/
int DidCollide4(MBITMAP image1, MBITMAP image2, int x1, int y1,
               int x2, int y2)
{
  int xstart, ystart, xend, yend;
  unsigned char c1, c2;
  register int x, y;
  int collision = 0;

  xstart = x1 - x2;
  ystart = y2 - y1;
  xend = image2.x;
  if(xend > (x1 + image1.x))
    xend = x1 + image1.x;
  yend = ystart + image2.y;
  if(yend > image1.y)
    yend = image1.y;

  if(xstart == xend || ystart == yend) return 0;

  y = ystart;
  do {
    x = xstart;
    do {
      c1 = image1.bits[(y*image1.x) + (x-xstart)];
      c2 = image2.bits[((y-ystart) * image2.x) + x];
      if(c1 != 255 && c2 != 255)
      {
        collision = 1;
        x = xend + 1;
        y = yend + 1;
      }
      x++;
     } while(x < xend);
    y++;
  } while(y < yend);

  return collision;
}

