#include <stdio.h>
#include <malloc.h>
#include <string.h>
#include <dpmi.h>
#include <conio.h>
#include <sys/movedata.h>
#include <pc.h>
#include <allegro.h>

#include "graph.h"

unsigned char masktable[8] = {0x80,0x40,0x20,0x10,0x08,0x04,0x02,0x01};
unsigned char bittable[8] = {0x01,0x02,0x04,0x08,0x10,0x20,0x40,0x80};

unsigned char *vidBuffer;
unsigned int yLookup[200];
unsigned char palette[768];
VIEW theView;

/*
   Puts the screen into graphics mode,
   gets memory for videoBuffer and
   generates a y-scan line lookup table.
   returns 1 on if got memory.
*/
int InitGraphics(void)
{
  int i;
  __dpmi_regs r;
  int gotMem = 0;

  r.x.ax = 0x0013;
  __dpmi_int(0x10, &r);

  for(i = 0; i < 200; i++)
  {
    yLookup[i] = i * 320;
  }

  /* Get memory for video buffer */
  vidBuffer = (unsigned char *)calloc(64000, 1);
  if(vidBuffer)
  {
    gotMem = 1;
  }
  return gotMem;
}

/*
    Changes the graphics screen to text mode and
    frees memory allocated by InitGraphics().
*/
void TermGraphics(void)
{
  __dpmi_regs r;

  r.x.ax = 0x0003;
  __dpmi_int(0x10, &r);

  if(vidBuffer)
  {
    free(vidBuffer);
  }
}

/*
   Sets a pixel in a video buffer.
*/
void PlotPixel(unsigned char *buffer, int x, int y, unsigned char color)
{
  *(buffer + x + yLookup[y]) = color;
}

/*
   Gets the color value of a pixel in a video buffer.
*/
unsigned char GetPixel(unsigned char *buffer, int x, int y)
{
  return *(buffer + x + yLookup[y]);
}

/*
   Clears a video buffer
*/
void ClearVidBuffer(unsigned char *buffer)
{
  memset(buffer, 0, 64000);
}

/*
   Displays a video buffer
*/
void DisplayVidBuffer(unsigned char *buffer)
{
  dosmemput(buffer, 64000, 0xA0000);
}

/*
   Copies an image to the video buffer.
*/
void BitBlit(unsigned char *source, unsigned char *dest, int xSize,
             int ySize, int xLoc, int yLoc)
{
  register int x, y;
  register int xMax, yMax;
  register int cnt = 0;
  xMax = xLoc + xSize;
  yMax = yLoc + ySize;
  y = yLoc;

  do {
    x = xLoc;
    do {
      if(x >= theView.xmin && x < theView.xmax &&
         y >= theView.ymin && y < theView.ymax)
      {
        *(dest + x + yLookup[y]) = *(source + cnt);
      }
      cnt++;
      x++;
    } while(x < xMax);
    y++;
  } while(y < yMax);
}

/*
   Copies an image to the video buffer.
*/
void BitBlit2(unsigned char *source, unsigned char *dest, int xSize,
             int ySize, int xLoc, int yLoc)
{
  register int x, y;
  register int xMax, yMax;
  register int cnt = 0;
  xMax = xLoc + xSize;
  yMax = yLoc + ySize;
  y = yLoc;

  do {
    x = xLoc;
    do {
      if(x >= 0 && x < 320 &&
         y >= 0 && y < 200)
      {
        *(dest + x + yLookup[y]) = *(source + cnt);
      }
      cnt++;
      x++;
    } while(x < xMax);
    y++;
  } while(y < yMax);
}

/*
   Copies an image with transparent areas to the
   video buffer.
*/
void TransBitBlit(unsigned char *source, unsigned char *dest, int xSize,
             int ySize, int xLoc, int yLoc, unsigned char trans)
{
  register int x, y;
  register int xMax, yMax;
  register int cnt = 0;

  xMax = xLoc + xSize;
  yMax = yLoc + ySize;
  y = yLoc;

  do {
    x = xLoc;
    do {
      if(x >= theView.xmin && x < theView.xmax &&
         y >= theView.ymin && y < theView.ymax)
      {
        if(*(source + cnt) != trans)
        {
          *(dest + x + yLookup[y]) = *(source + cnt);
        }
      }
      cnt++;
      x++;
    } while(x < xMax);
    y++;
  } while(y < yMax);
}

/*
   Copies one 64K buffer to another.
*/
void BufferCopy(unsigned char *source, unsigned char *dest)
{
  memcpy(dest, source, 64000);
}

/*
   Sets the palette.
*/
void SetPalette(void)
{
  int i;

  for(i = 0; i < 256; i++)
  {
    outportb(0x3C8, i);
    outportb(0x3C9, palette[i*3] >> 2);
    outportb(0x3C9, palette[i*3+1] >> 2);
    outportb(0x3C9, palette[i*3+2] >> 2);
  }
}

/*
   Finds the nearest palette entry for r, g, b
*/
int FindNearestColor(unsigned char r, unsigned char g, unsigned char b)
{
  long d1, Tempd;
  register int i = 0;
  register unsigned char r1, g1, b1;
  int ret;
  
  d1 = 200000;
  
  do {
    r1 = palette[i*3];
    g1 = palette[i*3+1];
    b1 = palette[i*3+2];
    
    Tempd = (r1-r)*(r1-r) + (g1-g)*(g1-g) + (b1-b)*(b1-b);
    if(Tempd < d1)
    {
      d1 = Tempd;
      ret = i;
    }
    i++;
  } while(i < 256);
  
  return ret;
}

/*
   Draws a line in the video buffer.
*/
void DrawLine(short x1, short y1, short x2, short y2, unsigned char color)
{
  register short LgDelta, ShDelta, Cycle, LgStep, ShStep;
  short swap;
  
  LgDelta = x2 - x1;
  ShDelta = y2 - y1;
  
  if(LgDelta < 0)
  {
    LgDelta = -LgDelta;
    LgStep = -1;
  } else
  {
    LgStep = 1;
  }
  
  if(ShDelta < 0)
  {
    ShDelta = -ShDelta;
    ShStep = -1;
  } else
  {
    ShStep = 1;
  }
  
  if(ShDelta < LgDelta)
  {
    Cycle = LgDelta >> 1;
    while(x1 != x2)
    {
      if(x1 >= theView.xmin && x2 < theView.xmax &&
         y1 >= theView.ymin && y2 < theView.ymax)
      {
        *(vidBuffer+yLookup[y1]+x1) = color;
      }
      Cycle += ShDelta;
      if(Cycle > LgDelta)
      {
        Cycle -= LgDelta;
        y1 += ShStep;
      }
      x1 += LgStep;
    }
    if(x1 >= theView.xmin && x2 < theView.xmax &&
       y1 >= theView.ymin && y2 < theView.ymax)
    {
      *(vidBuffer+yLookup[y1]+x1) = color;
    }
  } else
  {
    Cycle = ShDelta >> 1;
    swap = LgDelta;
    LgDelta = ShDelta;
    ShDelta = swap;
    swap = LgStep;
    LgStep = ShStep;
    ShStep = swap;
    while(y1 != y2)
    {
      if(x1 >= theView.xmin && x2 < theView.xmax &&
         y1 >= theView.ymin && y2 < theView.ymax)
      {
        *(vidBuffer+yLookup[y1]+x1) = color;
      }
      Cycle += ShDelta;
      if(Cycle > LgDelta)
      {
        Cycle -= LgDelta;
        x1 += ShStep;
      }
      y1 += LgStep;
    }
    if(x1 >= theView.xmin && x2 < theView.xmax &&
       y1 >= theView.ymin && y2 < theView.ymax)
    {
      *(vidBuffer+yLookup[y1]+x1) = color;
    }
  }
}

/*
   Draws a line in the video buffer. (Non clipped against view port)
*/
void DrawLine2(short x1, short y1, short x2, short y2, unsigned char color)
{
  register short LgDelta, ShDelta, Cycle, LgStep, ShStep;
  short swap;
  
  LgDelta = x2 - x1;
  ShDelta = y2 - y1;
  
  if(LgDelta < 0)
  {
    LgDelta = -LgDelta;
    LgStep = -1;
  } else
  {
    LgStep = 1;
  }
  
  if(ShDelta < 0)
  {
    ShDelta = -ShDelta;
    ShStep = -1;
  } else
  {
    ShStep = 1;
  }
  
  if(ShDelta < LgDelta)
  {
    Cycle = LgDelta >> 1;
    while(x1 != x2)
    {
      if(x1 >= 0 && x2 < 320 &&
         y1 >= 0 && y2 < 200)
      {
        *(vidBuffer+yLookup[y1]+x1) = color;
      }
      Cycle += ShDelta;
      if(Cycle > LgDelta)
      {
        Cycle -= LgDelta;
        y1 += ShStep;
      }
      x1 += LgStep;
    }
    if(x1 >= 0 && x2 < 320 &&
       y1 >= 0 && y2 < 200)
    {
      *(vidBuffer+yLookup[y1]+x1) = color;
    }
  } else
  {
    Cycle = ShDelta >> 1;
    swap = LgDelta;
    LgDelta = ShDelta;
    ShDelta = swap;
    swap = LgStep;
    LgStep = ShStep;
    ShStep = swap;
    while(y1 != y2)
    {
      if(x1 >= 0 && x2 < 320 &&
         y1 >= 0 && y2 < 200)
      {
        *(vidBuffer+yLookup[y1]+x1) = color;
      }
      Cycle += ShDelta;
      if(Cycle > LgDelta)
      {
        Cycle -= LgDelta;
        x1 += ShStep;
      }
      y1 += LgStep;
    }
    if(x1 >= 0 && x2 < 320 &&
       y1 >= 0 && y2 < 200)
    {
      *(vidBuffer+yLookup[y1]+x1) = color;
    }
  }
}

/*
   Draws a non-transparent scaled bitmap.
*/
void DrawScaledBitmap(int xsize, int ysize, int newx, int newy,
                      int xpos, int ypos, unsigned char *image)
{
  int *tempLookup;
  register Fixedpoint xcnt, ycnt, curx, cury;
  register int x, y;

  xcnt = FixedDiv(xsize, newx);
  ycnt = FixedDiv(ysize, newy);

  tempLookup = (int *)calloc(ysize, sizeof(int));
  if(!tempLookup) /* Can't get memory for lookup table.*/
  {
    /* Draw it the slower way */
    cury = 0;
    y = 0;
    do {
      curx = 0;
      x = 0;
      do {
        if(x+xpos >= theView.xmin && x+xpos < theView.xmax &&
           y+ypos >= theView.ymin && y+ypos < theView.ymax)
        {
          *(vidBuffer+x+xpos+yLookup[y+ypos]) = *(image + FIXED_TO_INT(curx)+
                                          FIXED_TO_INT(cury) * xsize);
        }
        x++;
        curx += xcnt;
      } while(x < newx);
      cury += ycnt;
      y++;
    } while(y < newy);
  } else
  {
    /* Draw it the faster way */
    y = 0;
    do {
      tempLookup[y] = y * xsize;
      y++;
    } while(y < ysize);

    cury = 0;
    y = 0;
    do {
      curx = 0;
      x = 0;
      do {
        if(x+xpos >= theView.xmin && x+xpos < theView.xmax &&
           y+ypos >= theView.ymin && y+ypos < theView.ymax)
        {
          *(vidBuffer+x+xpos+yLookup[y+ypos]) = *(image + FIXED_TO_INT(curx)+
                                          tempLookup[FIXED_TO_INT(cury)]);
        }
        x++;
        curx += xcnt;
      } while(x < newx);
      cury += ycnt;
      y++;
    } while(y < newy);

    free(tempLookup);
  }
}

/*
   Draws a transparent scaled bitmap.
*/
void TransDrawScaleBitmap(int xsize, int ysize, int newx, int newy,
                          int xpos, int ypos, unsigned char *image,
                          unsigned char trans)
{
  int *tempLookup;
  register Fixedpoint xcnt, ycnt, curx, cury;
  register int x, y;
  unsigned char c;

  xcnt = FixedDiv(xsize, newx);
  ycnt = FixedDiv(ysize, newy);

  tempLookup = (int *)calloc(ysize, sizeof(int));
  if(!tempLookup) /* Can't get memory for lookup table.*/
  {
    /* Draw it the slower way */
    cury = 0;
    y = 0;
    do {
      curx = 0;
      x = 0;
      do {
        if(x+xpos >= theView.xmin && x+xpos < theView.xmax &&
           y+ypos >= theView.ymin && y+ypos < theView.ymax)
        {
          c = *(image + FIXED_TO_INT(curx) + FIXED_TO_INT(cury) * xsize);
          if(c != trans)
          {
            *(vidBuffer + x + xpos + yLookup[y + ypos]) = c;
          }
        }
        x++;
        curx += xcnt;
      } while(x < newx);
      cury += ycnt;
      y++;
    } while(y < newy);
  } else
  {
    /* Draw it the faster way */
    y = 0;
    do {
      tempLookup[y] = y * xsize;
      y++;
    } while(y < ysize);

    cury = 0;
    y = 0;
    do {
      curx = 0;
      x = 0;
      do {
        if(x+xpos >= theView.xmin && x+xpos < theView.xmax &&
           y+ypos >= theView.ymin && y+ypos < theView.ymax)
        {
          c = *(image + FIXED_TO_INT(curx) + tempLookup[FIXED_TO_INT(cury)]);
          if(c != trans)
          {
            *(vidBuffer + x + xpos + yLookup[y + ypos]) = c;
          }
        }
        x++;
        curx += xcnt;
      } while(x < newx);
      cury += ycnt;
      y++;
    } while(y < newy);

    free(tempLookup);
  }
}

/*
   Sees if all the numbers are less than zero in a list.
*/
int IsGreaterThan(int list[], int numItems, int val)
{
  register int i = 0;
  int ret = 1;

  do {
    if(list[i] <= val)
    {
      ret = 0;
      i = numItems;
    }
    i++;
  } while(i < numItems);
  return ret;
}

/*
  Melts the screen into another.
*/
void Melt(unsigned char *newImage)
{
  int height[320];
  register int i, a = 0, k = 0;

  i = 0;
  do {
    height[i] = 1;
    i++;
  } while(i < 320);

  while(!IsGreaterThan(height, 320, 199))
  {
    i = 0;
    do {
      k = height[i];
      a = i;
      while(k--)
      {
        *(vidBuffer + a) = *(newImage + a);
        a += 320;
      }
      i++;
    } while(i < 320);

    DisplayVidBuffer(vidBuffer);

    i = 0;
    do {
      height[i] += random() % 8 + 4;
      i++;
    } while(i < 320);
  }
}

int PCXWriteLine(FILE *fp, unsigned char *p, int n)
{
  int i = 0, j = 0, t = 0;
  
  do {
    i = 0;
    while((p[t+i] == p[t+i+1]) && ((t+i) < n) && (i < 63))
    i++;
    if(i > 0)
    {
      if(fputc(i | 0xC0, fp) == EOF) return -1;
      if(fputc(p[t], fp) == EOF) return -1;
      t += i;
      j += 2;
    } else
    {
      if(((p[t]) & 0xC0) == 0xC0)
      {
        if(fputc(0xC1, fp) == EOF) return -1;
        j++;
      }
      if(fputc(p[t++], fp) == EOF) return -1;
      j++;
    }
  } while(t < n);
  return n;
}

void SavePCXFile(char *fname)
{
  int i;
  PCXHEAD pcx;
  FILE *fp;
  unsigned char *linebuffer;

  fp = fopen(fname, "wb");
  if(!fp)
  {
    return;
  }
      
  pcx.manufacturer = 10;
  pcx.encoding = 1;
  pcx.xmin = 0;
  pcx.ymin = 0;
  pcx.xmax = 320 - 1;
  pcx.ymax = 200 - 1;
  pcx.bits_per_pixel = 8;
  pcx.palette_type = 1;
  pcx.version = 5;
  pcx.color_planes = 1;
  pcx.bytes_per_line = 320;
  pcx.hres = 320;
  pcx.vres = 200;

  fwrite(&pcx, sizeof(PCXHEAD), 1, fp);

  linebuffer = vidBuffer;
  for(i = 0; i < 200; i++)
  {
    PCXWriteLine(fp, linebuffer, pcx.bytes_per_line);
    linebuffer += 320;
  }
  
  fputc(12, fp);
  fwrite(&palette, sizeof(char), 768, fp);
    
  fclose(fp);
}

