#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <allegro.h>

#include "space.h"
#include "misc.h"
#include "level.h"
#include "graph.h"


/* Variables */
LISTPTR listHead = NULL;


/*
   Reads all the level data
*/
void ReadLevelData(char *levelFile)
{
  char buffer[256];
  char parts[5][81];
  int curLine = 0;
  int curBadGuy = 0;
  int curMeteor = 0;
  FILE *fp;

  fp = fopen(levelFile, "r");
  if(!fp)
  {
    TermGraphics();
    printf("Error opening level file \'%s\'\n", levelFile);
    fflush(stdout);
    FreeMemory();
    allegro_exit();
    exit(1);
  }

  while(fgets(buffer, 81, fp))
  {
    if(buffer[0] != ';') /* this line is a comment */
    {
      sscanf(buffer, "%s %s %s %s %s\n", parts[0], parts[1], parts[2],
             parts[3], parts[4]);

      curLine++;

      if(!stricmp(parts[0], "USE_STARS"))
      {
        useStars = atoi(parts[2]);
        if(useStars < 0 || useStars > 1)
        {
          /* invalid USE_STARS */
          TermGraphics();
          printf("Level \'%s\'  Line: %d  ERROR: invalid value for "
                 "USE_STARS\n", levelFile, curLine);
          fflush(stdout);
          FreeMemory();
          allegro_exit();
          exit(1);
        }
      } else if(!stricmp(parts[0], "SCALE_BACK"))
      {
        scaleBack = atoi(parts[2]);
        if(scaleBack < 0 || scaleBack > 1)
        {
          /* invalid use of SCALE_BACK */
          TermGraphics();
          printf("Level \'%s\'  Line: %d  ERROR: invalid value for "
                 "SCALE_BACK\n", levelFile, curLine);
          fflush(stdout);
          FreeMemory();
          allegro_exit();
          exit(1);
        }
      } else if(!stricmp(parts[0], "SOLID_BACK"))
      {
        solidBack = atoi(parts[2]);
        if(solidBack < 0 || solidBack > 1)
        {
          /* invalid use of SOLID_BACK */
          TermGraphics();
          printf("Level \'%s\'  Line: %d  ERROR: invalid value for "
                 "SOLID_BACK\n", levelFile, curLine);
          fflush(stdout);
          FreeMemory();
          allegro_exit();
          exit(1);
        }
      } else if(!stricmp(parts[0], "BIG_BACK"))
      {
        whichBigBack = atoi(parts[2]);
        if(whichBigBack < 0 || whichBigBack >= NUM_BIGBACK)
        {
          /* invalid BIG_BACK */
          TermGraphics();
          printf("Level \'%s\'  Line: %d  ERROR: invalid value for "
                 "BIG_BACK\n", levelFile, curLine);
          fflush(stdout);
          FreeMemory();
          allegro_exit();
          exit(1);
        }
      } else if(!stricmp(parts[0], "LEVEL_DISTANCE"))
      {
        levelDistance = atoi(parts[2]);
        if(levelDistance < 1000)
        {
          /* level too short */
          TermGraphics();
          printf("Level \'%s\'  Line: %d  ERROR: level length too short.\n",
                 levelFile, curLine);
          fflush(stdout);
          FreeMemory();
          allegro_exit();
          exit(1);
        }
      } else if(!stricmp(parts[0], "MUST_KILL_BOSS"))
      {
        mustKillBoss = atoi(parts[2]);
        if(mustKillBoss < 0 || mustKillBoss > 1)
        {
          /* invalid MUST_KILL_BOSS */
          TermGraphics();
          printf("Level \'%s\'  Line: %d  ERROR: invalid value for "
                 "MUST_KILL_BOSS\n", levelFile, curLine);
          fflush(stdout);
          FreeMemory();
          allegro_exit();
          exit(1);
        }
      } else if(!stricmp(parts[0], "CAN_WARP"))
      {
        canWarp = atoi(parts[2]);
        if(canWarp < 0 || canWarp > 1)
        {
          /* invalid CAN_WARP */
          TermGraphics();
          printf("Level \'%s\'  Line: %d  ERROR: invalid value for "
                 "CAN_WARP\n", levelFile, curLine);
          fflush(stdout);
          FreeMemory();
          allegro_exit();
          exit(1);
        }
      } else if(!stricmp(parts[0], "ENEMY"))
      {
        if(curBadGuy >= MAX_BADGUYS)
        {
          /* too many bad guys */
          TermGraphics();
          printf("Level \'%s\'  Line: %d  ERROR: too many bad guys.\n",
                 levelFile, curLine);
          fflush(stdout);
          FreeMemory();
          allegro_exit();
          exit(1);
        }
        badGuyList[curBadGuy].type = atoi(parts[1]);
        badGuyList[curBadGuy].y = atoi(parts[2]);
        curBadGuy++;
      } else if(!stricmp(parts[0], "USE_METEOR"))
      {
        useMeteors = atoi(parts[2]);
        if(useMeteors < 0 || useMeteors > 1)
        {
          /* invalid CAN_WARP */
          TermGraphics();
          printf("Level \'%s\'  Line: %d  ERROR: invalid value for "
                 "USE_METEORS\n", levelFile, curLine);
          fflush(stdout);
          FreeMemory();
          allegro_exit();
          exit(1);
        }
      } else if(!stricmp(parts[0], "USE_BACKGROUND_IMAGE"))
      {
        useBackGroundImage = atoi(parts[2]);
        if(useBackGroundImage < 0 || useBackGroundImage > 1)
        {
          /* invalid USE_BACKGROUND_IMAGE */
          TermGraphics();
          printf("Level \'%s\'  Line: %d  ERROR: invalid value for "
                 "USE_BACKGROUND_IMAGE\n", levelFile, curLine);
          fflush(stdout);
          FreeMemory();
          allegro_exit();
          exit(1);
        }
      } else if(!stricmp(parts[0], "ADD_BAD_GUY_DIST"))
      {
        addBadGuyDistance = atoi(parts[2]);
        if(addBadGuyDistance < 0)
        {
          /* Invalid range */
          TermGraphics();
          printf("Level \'%s\'  Line: %d  ERROR: invalid value for "
                 "ADD_BAD_GUY_DIST\n", levelFile, curLine);
          fflush(stdout);
          FreeMemory();
          exit(1);
        }
      } else
      {
        /* Unknown command */
        TermGraphics();
        printf("Level \'%s\'  Line: %d  ERROR: unknown command\n",
               levelFile, curLine);
        fflush(stdout);
        FreeMemory();
        allegro_exit();
        exit(1);
      }
    }
  }
  fclose(fp);
}

/*
   Frees all the memory from the linked list
*/
void FreeListMem(void)
{
  LISTPTR curPtr, nextPtr;
  curPtr = listHead;
  
  while(curPtr)
  {
    nextPtr = curPtr->nextInfo;
    free(curPtr);
    curPtr = nextPtr;
  }
}

/*
   Adds an item to the linked list.
*/
LISTPTR AddToList(GROUPINFO gInfo, LISTPTR prevRec)
{
  LISTPTR newList = NULL;
  
  newList = (LISTPTR)malloc(sizeof(LIST));
  if(!newList)
  {
    printf("\nError allocating memory!\n");
    fflush(stdout);
    FreeMemory();
    exit(1);
  }
  
  strcpy(newList->gi.id, gInfo.id);
  newList->gi.offset = gInfo.offset;
  newList->gi.length = gInfo.length;
  newList->nextInfo = NULL;
  
  if(prevRec)
  {
    prevRec->nextInfo = newList;
  }
  
  return newList;
}

/*
   Finds a resource in the linked list.
*/
int FindResource(GROUPINFO *gInfo, char *id)
{
  int ret = 0;
  int found = 0;
  LISTPTR ptr = listHead;

  while(ptr != NULL && found == 0)
  {
    if(!stricmp(ptr->gi.id, id)) /* found match */
    {
      ret = 1;
      found = 1;
      strcpy(gInfo->id, ptr->gi.id);
      gInfo->offset = ptr->gi.offset;
      gInfo->length = ptr->gi.length;
    }
    ptr = ptr->nextInfo;
  }

  return ret;
}

/*
   Loads the linked list.
*/
void LoadLinkedList(char *fileName)
{
  long numEntries;
  FILE *fp;
  int i;
  GROUPINFO gInfo;
  LISTPTR ptr = NULL;

  fp = fopen(fileName, "rb");
  if(!fp)
  {
    printf("\nUnable to open \'%s\'.\n", fileName);
    fflush(stdout);
    FreeMemory();
    exit(1);
  }

  fread(&numEntries, sizeof(long), 1, fp);

  fread(&gInfo, sizeof(GROUPINFO), 1, fp);
  ptr = AddToList(gInfo, (LISTPTR)NULL);
  listHead = ptr;

  for(i = 1; i < numEntries; i++)
  {
    fread(&gInfo, sizeof(GROUPINFO), 1, fp);
    ptr = AddToList(gInfo, ptr);
  }

  fclose(fp);
}


/*
  The buying menu
*/
void BuyStuff(void)
{
  int curItem = 0;
  int k;
  int done = 0;

  do {
    ClearVidBuffer(vidBuffer);
    BitBlit2(selector.bits, vidBuffer, selector.x, selector.y, 0, 0);
    BitBlit2(buyPics[curItem].bits, vidBuffer, buyPics[curItem].x,
             buyPics[curItem].y, 20, 20);

    DisplayVidBuffer(vidBuffer);

    k = readkey();
    if((k & 0xFF) == 0x00 || (k & 0xFF) == 0xE0)
    {
      k >>= 8;

      if(k == 72) /* up arrow */
      {
        curItem++;
        if(curItem >= NUM_BUYPICS) curItem = 0;
      } else if(k == 80) /* down arrow */
      {
        curItem--;
        if(curItem < 0) curItem = NUM_BUYPICS - 1;
      }
    } else
    {
      k &= 0xFF;

      if(k == 27 || k == 'q' || k == 'Q') /* ESC key */
      {
        done = 1;
      } if(k == 'n' || k == 'N')
      {
        curItem++;
        if(curItem >= NUM_BUYPICS) curItem = 0;
      } if(k == 'b' || k == 'B')
      {
        curItem--;
        if(curItem < 0) curItem = NUM_BUYPICS - 1;
      }
    }
  } while(!done);
}

