#include <stdio.h>
#include <malloc.h>
#include <string.h>
#include <stdlib.h>
#include <allegro.h>

#include "space.h"
#include "graph.h"
#include "misc.h"
#include "level.h"

/*
    Initializes important variables and loads
    graphics.
*/
void InitVariables(void)
{
  FILE *fp;
  int i, r, cnt;
  char buffer[81];
  GROUPINFO gInfo;

  /* Setup variables for the player */
  me.x = 160;
  me.y = 100;
  me.action = MOVE;
  me.frame = 0;
  me.weapons = 1 + 2 + 4 + 8;
  me.speed = 2;
  me.numMissles = 3;
  me.weaponPower = me.maxWeaponPower = 100;
  me.drivePower = me.maxDrivePower = 500;
  me.disTravel = 0;
  me.score = 0;
  me.health = 5;
  me.curLaser = 0;

  /* setup view variables */
  theView.xmin = 0; theView.xmax = 320;
  theView.ymin = 10; theView.ymax = 200;

  /* other variables */
  scaleBack = 0;
  useStars = 0;
  solidBack = 1;
  whichBigBack = 1;
  levelDistance = 10000;
  mustKillBoss = 1;
  bossOnScreen = 0;
  canWarp = 0;
  useMeteors = 1;
  useBackGroundImage = 1;

  fp = fopen("data\\space.dat", "rb");
  if(!fp)
  {
    printf("\nCan't open \'data\\space.dat\'.\n");
    fflush(stdout);
    FreeMemory();
    exit(1);
  }

  /* Load ships */
  for(i = 0; i < NUM_SHIPS; i++)
  {
    sprintf(buffer, "ship%d", i+1);
    if(!FindResource(&gInfo, buffer))
    {
      printf("\nCan't find resource \'%s\'\n", buffer);
      fflush(stdout);
      fclose(fp);
      FreeMemory();
      exit(1);
    }
    fseek(fp, gInfo.offset, SEEK_SET);
    fread(&ship[i].x, 2, 1, fp);
    fread(&ship[i].y, 2, 1, fp);

    ship[i].bits = (unsigned char *)malloc(ship[i].x * ship[i].y);
    if(!ship[i].bits)
    {
      printf("\nCan't get memory.\n");
      fflush(stdout);
      fclose(fp);
      FreeMemory();
      exit(1);
    }
    fread(ship[i].bits, 1, ship[i].x * ship[i].y, fp);
  }

  /* Load rocks */
  for(i = 0; i < NUM_ROCKS; i++)
  {
    sprintf(buffer, "rock%d", i+1);
    if(!FindResource(&gInfo, buffer))
    {
      printf("\nCan't find resource \'%s\'\n", buffer);
      fflush(stdout);
      fclose(fp);
      FreeMemory();
      exit(1);
    }
    fseek(fp, gInfo.offset, SEEK_SET);
    fread(&rocks[i].x, 2, 1, fp);
    fread(&rocks[i].y, 2, 1, fp);

    rocks[i].bits = (unsigned char *)malloc(rocks[i].x * rocks[i].y);
    if(!rocks[i].bits)
    {
      printf("\nCan't get memory.\n");
      fflush(stdout);
      fclose(fp);
      FreeMemory();
      exit(1);
    }
    fread(rocks[i].bits, 1, rocks[i].x * rocks[i].y, fp);
  }

  /* Load laser bitmaps */
  for(i = 0; i < NUM_LASERS; i++)
  {
    sprintf(buffer, "laser%d", i);
    if(!FindResource(&gInfo, buffer))
    {
      printf("\nCan't find resource \'%s\'\n", buffer);
      fflush(stdout);
      fclose(fp);
      FreeMemory();
      exit(1);
    }
    fseek(fp, gInfo.offset, SEEK_SET);
    fread(&lasers[i].x, 2, 1, fp);
    fread(&lasers[i].y, 2, 1, fp);

    lasers[i].bits = (unsigned char *)malloc(lasers[i].x*lasers[i].y);
    if(!lasers[i].bits)
    {
      printf("\nCan't get memory.\n");
      fflush(stdout);
      fclose(fp);
      FreeMemory();
      exit(1);
    }
    fread(lasers[i].bits, 1, lasers[i].x*lasers[i].y, fp);
  }

  /* Load background images */
  for(i = 0; i < NUM_BACKGROUND; i++)
  {
    sprintf(buffer, "back%d", i+1);
    if(!FindResource(&gInfo, buffer))
    {
      printf("\nCan't find resource \'%s\'\n", buffer);
      fflush(stdout);
      fclose(fp);
      FreeMemory();
      exit(1);
    }
    fseek(fp, gInfo.offset, SEEK_SET);
    fread(&background[i].x, 2, 1, fp);
    fread(&background[i].y, 2, 1, fp);

    background[i].bits =
        (unsigned char *)malloc(background[i].x * background[i].y);
    if(!background[i].bits)
    {
      printf("\nCan't get memory.\n");
      fflush(stdout);
      fclose(fp);
      FreeMemory();
      exit(1);
    }
    fread(background[i].bits, 1, background[i].x * background[i].y, fp);
  }

  /* Load big background images */
  for(i = 0; i < NUM_BIGBACK; i++)
  {
    sprintf(buffer, "bigback%d", i);
    if(!FindResource(&gInfo, buffer))
    {
      printf("\nCan't find resource \'%s\'\n", buffer);
      fflush(stdout);
      fclose(fp);
      FreeMemory();
      exit(1);
    }
    fseek(fp, gInfo.offset, SEEK_SET);
    fread(&bigBack[i].x, 2, 1, fp);
    fread(&bigBack[i].y, 2, 1, fp);

    bigBack[i].bits = (unsigned char *)malloc(bigBack[i].x * bigBack[i].y);
    if(!bigBack[i].bits)
    {
      printf("\nCan't get memory.\n");
      fflush(stdout);
      fclose(fp);
      FreeMemory();
      exit(1);
    }
    fread(bigBack[i].bits, 1, bigBack[i].x * bigBack[i].y, fp);
  }

  /* Load explosion images */
  for(i = 0; i < NUM_EXP_FRAMES; i++)
  {
    sprintf(buffer, "exp%d", i+1);
    if(!FindResource(&gInfo, buffer))
    {
      printf("\nCan't find resource \'%s\'\n", buffer);
      fflush(stdout);
      fclose(fp);
      FreeMemory();
      exit(1);
    }
    fseek(fp, gInfo.offset, SEEK_SET);
    fread(&exp[i].x, 2, 1, fp);
    fread(&exp[i].y, 2, 1, fp);

    exp[i].bits = (unsigned char *)malloc(exp[i].x * exp[i].y);
    if(!exp)
    {
      printf("\nCan't get memory.\n");
      fflush(stdout);
      fclose(fp);
      FreeMemory();
      exit(1);
    }
    fread(exp[i].bits, 1, (exp[i].x * exp[i].y), fp);
  }

  /* Load number images */
  for(i = 0; i < 10; i++)
  {
    sprintf(buffer, "%d", i);
    if(!FindResource(&gInfo, buffer))
    {
      printf("\nCan't find resource \'%s\'\n", buffer);
      fflush(stdout);
      fclose(fp);
      FreeMemory();
      exit(1);
    }
    fseek(fp, gInfo.offset, SEEK_SET);
    fread(&numbers[i].x, 2, 1, fp);
    fread(&numbers[i].y, 2, 1, fp);

    numbers[i].bits = (unsigned char *)malloc(numbers[i].x * numbers[i].y);
    if(!numbers[i].bits)
    {
      printf("\nCan't get memory.\n");
      fflush(stdout);
      fclose(fp);
      FreeMemory();
      exit(1);
    }
    fread(numbers[i].bits, 1, numbers[i].x * numbers[i].y, fp);
  }

  /* Load health bar */
  sprintf(buffer, "health1", i);
  if(!FindResource(&gInfo, buffer))
  {
    printf("\nCan't find resource \'%s\'\n", buffer);
    fflush(stdout);
    fclose(fp);
    FreeMemory();
    exit(1);
  }
  fseek(fp, gInfo.offset, SEEK_SET);
  fread(&health1.x, 2, 1, fp);
  fread(&health1.y, 2, 1, fp);
  health1.bits = (unsigned char *)malloc(health1.x * health1.y);
  if(!health1.bits)
  {
    printf("\nCan't get memory.\n");
    fflush(stdout);
    fclose(fp);
    FreeMemory();
    exit(1);
  }
  fread(health1.bits, 1, health1.x * health1.y, fp);

  /* Load enemies */
  /* sphere fighter */
  for(i = 0; i < 2; i++)
  {
    sprintf(buffer, "enemy%d-0", i);
    if(!FindResource(&gInfo, buffer))
    {
      printf("\nCan't find resource \'%s\'\n", buffer);
      fflush(stdout);
      fclose(fp);
      FreeMemory();
      exit(1);
    }
    fseek(fp, gInfo.offset, SEEK_SET);
    fread(&enemies[i].x, 2, 1, fp);
    fread(&enemies[i].y, 2, 1, fp);
    enemies[i].bits = (unsigned char *)malloc(enemies[i].x*enemies[i].y);
    if(!enemies[i].bits)
    {
      printf("\nCan't get memory.\n");
      fflush(stdout);
      fclose(fp);
      FreeMemory();
      exit(1);
    }
    fread(enemies[i].bits, 1, enemies[i].x*enemies[i].y, fp);
  }

  /* Load boss images */
  cnt = 0;
  for(i = 0; i < 2; i++)
  {
    for(r = 0; r < 2; r++)
    {
      sprintf(buffer, "boss%d-%d", i+1, r);
      if(!FindResource(&gInfo, buffer))
      {
        printf("\nCan't find resource \'%s\'\n", buffer);
        fflush(stdout);
        fclose(fp);
        FreeMemory();
        exit(1);
      }
      fseek(fp, gInfo.offset, SEEK_SET);
      fread(&enemies[cnt+2].x, 2, 1, fp);
      fread(&enemies[cnt+2].y, 2, 1, fp);
      enemies[cnt+2].bits =
         (unsigned char *)malloc(enemies[cnt+2].x*enemies[cnt+2].y);
      if(!enemies[cnt+2].bits)
      {
        printf("\nCan't get memory.\n");
        fflush(stdout);
        fclose(fp);
        FreeMemory();
        exit(1);
      }
      fread(enemies[cnt+2].bits, 1, enemies[cnt+2].x*enemies[cnt+2].y, fp);
    }
  }

  /* Load powerup images */
  for(i = 0; i < NUM_POWER_UPS; i++)
  {
    sprintf(buffer, "pow%d", i);
    if(!FindResource(&gInfo, buffer))
    {
      printf("\nCan't find resource \'%s\'\n", buffer);
      fflush(stdout);
      fclose(fp);
      FreeMemory();
      exit(1);
    }
    fseek(fp, gInfo.offset, SEEK_SET);
    fread(&powerUps[i].x, 2, 1, fp);
    fread(&powerUps[i].y, 2, 1, fp);

    powerUps[i].bits = (unsigned char *)malloc(powerUps[i].x*powerUps[i].y);
    if(!powerUps[i].bits)
    {
      printf("\nCan't get memory.\n");
      fflush(stdout);
      fclose(fp);
      FreeMemory();
      exit(1);
    }
    fread(powerUps[i].bits, 1, powerUps[i].x*powerUps[i].y, fp);
  }

  /* Load weapon icons */
  for(i = 0; i < MAX_WEAPON_ICONS; i++)
  {
    sprintf(buffer, "wicon%d", i);
    if(!FindResource(&gInfo, buffer))
    {
      printf("\nCan't find resource \'%s\'\n", buffer);
      fflush(stdout);
      fclose(fp);
      FreeMemory();
      exit(1);
    }
    fseek(fp, gInfo.offset, SEEK_SET);
    fread(&weaponIcon[i].x, 2, 1, fp);
    fread(&weaponIcon[i].y, 2, 1, fp);

    weaponIcon[i].bits =
       (unsigned char *)malloc(weaponIcon[i].x*weaponIcon[i].y);
    if(!weaponIcon[i].bits)
    {
      printf("\nCan't get memory.\n");
      fflush(stdout);
      fclose(fp);
      FreeMemory();
      exit(1);
    }
    fread(weaponIcon[i].bits, 1, weaponIcon[i].x*weaponIcon[i].y, fp);
  }

  /* Load Missles */
  for(i = 0; i < NUM_MISSLES; i++)
  {
    sprintf(buffer, "missle%d", i+1);
    if(!FindResource(&gInfo, buffer))
    {
      printf("\nCan't find resource \'%s\'\n", buffer);
      fflush(stdout);
      fclose(fp);
      FreeMemory();
      exit(1);
    }
    fseek(fp, gInfo.offset, SEEK_SET);
    fread(&missle[i].x, 2, 1, fp);
    fread(&missle[i].y, 2, 1, fp);

    missle[i].bits = (unsigned char *)malloc(missle[i].x*missle[i].y);
    if(!missle[i].bits)
    {
      printf("\nCan't get memory.\n");
      fflush(stdout);
      fclose(fp);
      FreeMemory();
      exit(1);
    }
    fread(missle[i].bits, 1, missle[i].x*missle[i].y, fp);
  }

  /* Load enemy lasers */
  for(i = 0; i < NUM_ENLASERS; i++)
  {
    sprintf(buffer, "enlas%d", i);
    if(!FindResource(&gInfo, buffer))
    {
      printf("\nCan't find resource \'%s\'\n", buffer);
      fflush(stdout);
      fclose(fp);
      FreeMemory();
      exit(1);
    }
    fseek(fp, gInfo.offset, SEEK_SET);
    fread(&enLasers[i].x, 2, 1, fp);
    fread(&enLasers[i].y, 2, 1, fp);

    enLasers[i].bits = (unsigned char *)malloc(enLasers[i].x*enLasers[i].y);
    if(!enLasers[i].bits)
    {
      printf("\nCan't get memory.\n");
      fflush(stdout);
      fclose(fp);
      FreeMemory();
      exit(1);
    }
    fread(enLasers[i].bits, 1, enLasers[i].x*enLasers[i].y, fp);
  }

  /* Load images you can buy */
  for(i = 0; i < NUM_BUYPICS; i++)
  {
    sprintf(buffer, "buy%d", i);
    if(!FindResource(&gInfo, buffer))
    {
      printf("\nCan't find resource \'%s\'\n", buffer);
      fflush(stdout);
      fclose(fp);
      FreeMemory();
      exit(1);
    }
    fseek(fp, gInfo.offset, SEEK_SET);
    fread(&buyPics[i].x, 2, 1, fp);
    fread(&buyPics[i].y, 2, 1, fp);

    buyPics[i].bits = (unsigned char *)malloc(buyPics[i].x*buyPics[i].y);
    if(!buyPics[i].bits)
    {
      printf("\nCan't get memory.\n");
      fflush(stdout);
      fclose(fp);
      FreeMemory();
      exit(1);
    }
    fread(buyPics[i].bits, 1, buyPics[i].x*buyPics[i].y, fp);
  }

  sprintf(buffer, "SELECT0", i);
  if(!FindResource(&gInfo, buffer))
  {
    printf("\nCan't find resource \'%s\'\n", buffer);
    fflush(stdout);
    fclose(fp);
    FreeMemory();
    exit(1);
  }
  fseek(fp, gInfo.offset, SEEK_SET);
  fread(&selector.x, 2, 1, fp);
  fread(&selector.y, 2, 1, fp);

  selector.bits = (unsigned char *)malloc(selector.x*selector.y);
  if(!selector.bits)
  {
    printf("\nCan't get memory.\n");
    fflush(stdout);
    fclose(fp);
    FreeMemory();
    exit(1);
  }
  fread(selector.bits, 1, selector.x*selector.y, fp);

  /* close data\space.dat */
  fclose(fp);

  InitLists();
}

/*
   Clears all info on meteors, stars, enemies, etc.
*/
void InitLists(void)
{
  /* Initialize some variables */
  me.health = 5;
  me.speed = 2;
  me.numMissles = 3;
  me.weaponPower = me.maxWeaponPower = 100;
  me.drivePower = me.maxDrivePower = 500;
  me.disTravel = 0;
  me.x = 160;
  me.y = 100;
  me.action = MOVE;
  me.frame = 0;

  /* Make stars */
  for(i = 0; i < NUM_STARS; i++)
  {
    stars[i].x = (random() % (theView.xmax - theView.xmin))+theView.xmin;
    stars[i].y = (random() % (theView.ymax - theView.ymin))+theView.ymin;
    if(i <= 19)
    {
      stars[i].z = 1;
    } else if(i <= 39)
    {
      stars[i].z = 2;
    } else
    {
      stars[i].z = 3;
    }
  }

  /* Make Meteors */
  for(i = 0; i < NUM_METEORS; i++)
  {
    meteors[i].z = random() % 2;
	 meteors[i].x = theView.xmin-(random()	% 50);
	 meteors[i].y = (random()%(theView.ymax-theView.ymin))+theView.ymin;
	 meteors[i].remake = 1;
  }

  /* clear all shots */
  for(i = 0; i < MAX_SHOTS; i++)
  {
    shots[i].type = -1;
  }

  /* Clear out all explosions */
  for(i = 0; i < MAX_EXPLOSIONS; i++)
  {
    explosions[i].type = -1;
  }

  /* clear all power ups */
  for(i = 0; i < MAX_POWER_UPS; i++)
  {
    powerUpInfo[i].z = -1;
  }

  /* clear bad guys on screen */
  for(i = 0; i < NUM_BADONSCREEN; i++)
  {
    badGuys[i].action = -1;
  }

  /* clear all enemies */
  for(i = 0; i < MAX_BADGUYS; i++)
  {
    badGuyList[i].type = -1;
  }
}

/*
   Frees all the allocated memory.
*/
void FreeMemory(void)
{
  int r;

  for(r = 0; r < NUM_SHIPS; r++)
  {
    if(ship[r].bits)
    {
      free(ship[r].bits);
    }
  }
  for(r = 0; r < NUM_ROCKS; r++)
  {
    if(rocks[r].bits)
    {
      free(rocks[r].bits);
    }
  }
  for(r = 0; r < NUM_BACKGROUND; r++)
  {
    if(background[r].bits)
    {
      free(background[r].bits);
    }
  }
  for(r = 0; r < NUM_LASERS; r++)
  {
    if(lasers[r].bits)
    {
      free(lasers[r].bits);
    }
  }
  for(r = 0; r < NUM_EXP_FRAMES; r++)
  {
    if(exp[r].bits)
    {
      free(exp[r].bits);
    }
  }
  for(r = 0; r < 10; r++)
  {
    if(numbers[r].bits)
    {
      free(numbers[r].bits);
    }
  }
  if(health1.bits)
  {
    free(health1.bits);
  }
  for(r = 0; r < NUM_ENEMIES; r++)
  {
    if(enemies[r].bits)
    {
      free(enemies[r].bits);
    }
  }
  for(r = 0; r < NUM_BIGBACK; r++)
  {
    if(bigBack[0].bits)
    {
      free(bigBack[0].bits);
    }
  }
  for(r = 0; r < MAX_WEAPON_ICONS; r++)
  {
    if(weaponIcon[r].bits)
    {
      free(weaponIcon[r].bits);
    }
  }
  for(r = 0; r < NUM_MISSLES; r++)
  {
    if(missle[r].bits)
    {
      free(missle[r].bits);
    }
  }
  for(r = 0; r < NUM_POWER_UPS; r++)
  {
    if(powerUps[r].bits)
    {
      free(powerUps[r].bits);
    }
  }
  for(r = 0; r < NUM_ENLASERS; r++)
  {
    if(enLasers[r].bits)
    {
      free(enLasers[r].bits);
    }
  }
  for(r = 0; r < NUM_BUYPICS; r++)
  {
    if(buyPics[r].bits)
    {
      free(buyPics[r].bits);
    }
  }

  if(selector.bits)
  {
    free(selector.bits);
  }

  if(useSound)
  {
    if(item[0])
      destroy_midi((MIDI *)item[0]);
    if(item[1])
      destroy_midi((MIDI *)item[1]);
    if(item[2])
      destroy_sample((SAMPLE *)item[2]);
    if(item[3])
      destroy_midi((MIDI *)item[3]);
    if(item[4])
      destroy_sample((SAMPLE *)item[4]);
    if(item[5])
      destroy_sample((SAMPLE *)item[5]);
    if(item[6])
      destroy_sample((SAMPLE *)item[6]);
    if(item[7])
      destroy_sample((SAMPLE *)item[7]);
  }

  FreeListMem();
}

/*
   Searches through a list of arguments to see if
   a certain one exists.
*/
int IsParam(int numArgs, char *args[], char *find)
{
  int i;
  int ret = 0;

  for(i = 1; i < numArgs; i++)
  {
    if(!stricmp(args[i], find))
    {
      ret = i;
    }
  }

  return ret;
}

/*
   Loads a palette.
*/
void LoadPalette(char *pal)
{
  FILE *fp;
  GROUPINFO gInfo;

  /* Load palette */
  fp = fopen("data\\space.dat", "rb");
  if(!fp)
  {
    TermGraphics();
    printf("\nCan't open \'data\\space.dat\'.\n");
    FreeMemory();
    fflush(stdout);
    allegro_exit();
    exit(1);
  }
  if(!FindResource(&gInfo, pal))
  {
    TermGraphics();
    printf("\nCan't find resource \'%s\'\n", pal);
    fflush(stdout);
    fclose(fp);
    FreeMemory();
    allegro_exit();
    exit(1);
  }
  fseek(fp, gInfo.offset, SEEK_SET);
  fread(palette, 1, 768, fp);
  fclose(fp);
}


/*
   Loads the setup file.
*/
void LoadSetupFile(void)
{
  char buffer[256];
  char part[2][81];
  FILE *fp;

  fp = fopen("space.cfg", "r");
  if(!fp)
  {
    printf("\nError opening \'space.cfg\'\n");
    exit(1);
  }

  while(fgets(buffer, 255, fp))
  {
    sscanf(buffer, "%s = %s", part[0], part[1]);

    if(!stricmp(part[0], "DIGI_CARD")) /* the type of digital card */
    {
      digicard = atoi(part[1]);
    } else if(!stricmp(part[0], "MIDI_CARD")) /* type of music card */
    {
      midicard = atoi(part[1]);
    } else if(!stricmp(part[0], "VOL")) /* volume of sounds */
    {
      vol = atoi(part[1]);
    } else if(!stricmp(part[0], "FREQ")) /* freq of sounds */
    {
      freq = atoi(part[1]);
    } else if(!stricmp(part[0], "PAN")) /* pan of sounds */
    {
      pan = atoi(part[1]);
    } else if(!stricmp(part[0], "INTRO_MUS")) /* intro music */
    {
      strcpy(soundFiles[3], part[1]);
    } else if(!stricmp(part[0], "WARP_MUS")) /* music in warp */
    {
      strcpy(soundFiles[1], part[1]);
    } else if(!stricmp(part[0], "NORMAL_MUS")) /* normal level music */
    {
      strcpy(soundFiles[0], part[1]);
    } else if(!stricmp(part[0], "LASER1_SND")) /* sound of laser */
    {
      strcpy(soundFiles[2], part[1]);
    } else if(!stricmp(part[0], "EXPLODE1_SND")) /* sound of explosion #1 */
    {
      strcpy(soundFiles[4], part[1]);
    } else if(!stricmp(part[0], "MISSLE1_SND")) /* sound of missle #1 */
    {
      strcpy(soundFiles[5], part[1]);
    } else if(!stricmp(part[0], "POWERUP_SND")) /* sound of powerup */
    {
      strcpy(soundFiles[6], part[1]);
    } else if(!stricmp(part[0], "ENEMY_LASER_SND")) /* sound of enemy shot */
    {
      strcpy(soundFiles[7], part[1]);
    }

  }

  fclose(fp);
}

/*
   Adds an explosion.
*/
void AddExplosion(int x, int y, int type, int delay)
{
  register int i = 0;

  do {
    if(explosions[i].type == -1)
    {
      explosions[i].type = type;
      explosions[i].x = x;
      explosions[i].y = y;
      explosions[i].wait = 0;
      explosions[i].delay = delay;
      explosions[i].frame = 0;
      i = NUM_EXP_FRAMES + 1;
    }
    i++;
  } while(i < NUM_EXP_FRAMES);
}

/*
   Returns the damage that a certain weapon causes.
*/
int WeaponDamage(int type)
{
  int ret = 1;

  switch(type)
  {
    case 0:  /* normal laser */
             ret = 1; break;
    case 1:  /* mega laser */
             ret = 2; break;
    case 2:  /* particle cannon */
             ret = 1; break;
    case 3:  /* plasma cannon */
             ret = 4; break;
    case 4:  /* missle */
             ret = 10; break;
  }

  return ret;
}

/*
  Returns 1 if the weapon is possesed.
*/
int HaveWeapon(int type, int weapons)
{
  int ret = 0;
  int temp;
  register int i;
  int cnt;

  temp = 1;
  for(i = 0; i < type; i++)
  {
    temp *= 2;
  }

  if(weapons & temp)
  {
    ret = 1;
  }

  return ret;
}

/*
   Makes a power up item.
*/
void AddPowerUp(int type, int x, int y)
{
  register int i = 0;

  do {
    if(powerUpInfo[i].z == -1)
    {
      powerUpInfo[i].x = x;
      powerUpInfo[i].y = y;
      powerUpInfo[i].z = type;
      i = MAX_POWER_UPS;
    }
    i++;
  } while(i < MAX_POWER_UPS);
}

/*
  Adds the power up effect.
*/
void UsePowerUp(int type)
{
  int temp;

  switch(type)
  {
     case 0:  /* shield restore */
              me.health++;
              if(me.health > 5) me.health = 5;
              break;
     case 1:  /* restore weapon energy */
              me.weaponPower = me.maxWeaponPower;
              break;
     case 2:  /* restore all warp energy */
              me.drivePower = me.maxDrivePower;
              break;
     case 3:  /* make ship invincable for short time. */

              break;
     case 4:  /* Money bonus */
              temp = (random() % 1900) + 10;
              me.score += temp;
              pointCount += temp;
              break;
     case 5:  /* another missle */
              me.numMissles++;
              if(me.numMissles > 20) me.numMissles = 20;
              break;
  }
}

/*
   Creates a standard explosion.
*/
void StandardExplosion(int x, int y, int type, int delayFactor)
{
  AddExplosion(x, y, type, 0 * delayFactor);
  AddExplosion(x - 1, y + 1, type, 2 * delayFactor);
  AddExplosion(x + 1, y - 1, type, 4 * delayFactor);
  AddExplosion(x + 1, y, type, 6 * delayFactor);
  AddExplosion(x + 3, y - 3, type, 8 * delayFactor);
}

/*
   Shows the players ship exploding.
*/
void ShowPlayerDie(void)
{
  long t, waitTime;
  register int cntr;

  theView.xmin = 0; theView.xmax = 320;
  theView.ymin = 0; theView.ymax = 200;

  for(cntr = 0; cntr < 180; cntr++)
  {
     t = rawclock();
     waitTime = t + 1;

     ClearVidBuffer(vidBuffer);

     TransBitBlit(ship[0].bits, vidBuffer, ship[0].x * (10 + cntr/18),
                 ship[0].y * (10 + cntr/18), 0, 0, 255);

     DisplayVidBuffer(vidBuffer);
     while(waitTime > rawclock()) { }
  }

  theView.xmin = 0; theView.xmax = 320;
  theView.ymin = 10; theView.ymax = 200;
}

/*
   Checks to see if a file exists.
   Return 1 if it does or 0 if it
   doesn't.
*/
int FileExist(char *fname)
{
  FILE *fp;
  int ret = 0;

  fp = fopen(fname)
  if(fp)
  {
    ret = 1;
    fclose(fp);
  }
  return ret;
}


