#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <conio.h>
#include <time.h>
#include <dpmi.h>
#include <allegro.h>

#include "graph.h"
#include "space.h"
#include "collide.h"
#include "misc.h"
#include "mouse.h"
#include "level.h"
#include "badguy.h"


/* Assembly language functions */
extern void WaitRetrace(void);

/* Global variables */
STAR stars[NUM_STARS];
STAR meteors[NUM_METEORS];
STAR powerUpInfo[MAX_POWER_UPS];
EXPLOSION explosions[MAX_EXPLOSIONS];
PLAYER me;
PLAYER badGuys[NUM_BADONSCREEN];
BADGUYLIST badGuyList[MAX_BADGUYS];
MBITMAP ship[NUM_SHIPS];
MBITMAP rocks[NUM_ROCKS];
MBITMAP background[NUM_BACKGROUND];
MBITMAP lasers[NUM_LASERS];
MBITMAP enLasers[NUM_ENLASERS];
MBITMAP exp[NUM_EXP_FRAMES];
MBITMAP enemies[NUM_ENEMIES];
MBITMAP numbers[10];
MBITMAP health1;
MBITMAP missle[NUM_MISSLES];
MBITMAP bigBack[NUM_BIGBACK];
MBITMAP weaponIcon[MAX_WEAPON_ICONS];
MBITMAP powerUps[NUM_POWER_UPS];
MBITMAP buyPics[NUM_BUYPICS];
MBITMAP selector;
SHOT shots[MAX_SHOTS];
SHOT enemyShots[MAX_SHOTS];
void *item[9];
char soundFiles[9][81] = {"music\\mus1.mid", "music\\mus2.mid",
                          "sound\\laser.wav", "music\\intro.mid",
                          "sound\\explode.wav", "sound\\rocket.wav",
                          "sound\\powerup.wav", "sound\\laser3.wav"};
int useSound = 1;
int digicard = 4;
int midicard = 4;
int vol = 255;
int freq = 1000;
int pan = 128;
int useStars, scaleBack, solidBack;
int whichBigBack;
int levelDistance;
int mustKillBoss;
int bossOnScreen;
int canWarp;
int useMeteors;
int pointCount;
int useBackGroundImage;
int addBadGuyDistance;
int useMouse;
int godMode = 0;

/* functions */
int MainLoop(char *levelName);
void LoadSounds(void);


void main(int argc, char *argv[])
{
  FILE *fp;
  int k, doneGame = 0, ret;
  char levelName[81];
  GROUPINFO gInfo;

  srandom(45);

  printf("SPACE COMBAT   v0.60  DEMO VERSION\n");
  printf("Loading please wait...");
  fflush(stdout);
  LoadLinkedList("data\\space.dat");
  LoadSetupFile();
  LoadSounds();
  InitVariables();

  /* check command line parameters */
  if(IsParam(argc, argv, "-nosound"))
  {
    useSound = 0;
  }
  if(IsParam(argc, argv, "-godmode"))
  {
    godMode = 1;
  }

  useMouse = CheckMouseDriver();

  i_love_bill = TRUE; /* makes game run better under windows */
  allegro_init();
  install_timer();
  install_keyboard();

  if(useSound == 1)
  {
    if(install_sound(digicard, midicard, argv[0]) != 0)
    {
      printf("\nError initialising sound system\n%s\n",
              allegro_error);
      printf("No sounds will be used.\n");
      fflush(stdout);
      k = readkey();
      useSound = 0;
    }      else
    {
      useSound = 1;
    }
  }

  printf("\n\n");
  printf("\nObject of game: To score as many points as possible by shooting");
  printf("\n                meteors and badguys.");
  printf("\nKeys:");
  printf("\n   Arrow keys or mouse to move.");
  printf("\n   +, -   to speed up or slow down");
  printf("\n   <space bar> or left mouse button to fire laser");
  printf("\n   <enter> or right mouse button to fire missles");
  printf("\n   <esc> to quit");
  printf("\n   <F10> take PCX picture of screen in SPACE.PCX");
  printf("\n\nPress any key to continue.");
  fflush(stdout);
  k = readkey();

  if(InitGraphics())
  {
    /* Display title screen */
    LoadPalette("palette2");
    SetPalette();
    fp = fopen("data\\space.dat", "rb");
    if(!fp)
    {
      printf("\nCan't open \'data\\space.dat\'\n");
      fflush(stdout);
      FreeMemory();
      allegro_exit();
      exit(1);
    }
    if(!FindResource(&gInfo, "title"))
    {
      printf("\nCan't find resource \'title\'\n");
      fflush(stdout);
      fclose(fp);
      FreeMemory();
      allegro_exit();
      exit(1);
    }
    fseek(fp, gInfo.offset+4, SEEK_SET);
    fread(vidBuffer, 1, 64000, fp);
    fclose(fp);
    DisplayVidBuffer(vidBuffer);
    if(useSound)
    {
      play_midi((MIDI *)item[3], TRUE);
    }
    k = readkey();

    ClearVidBuffer(vidBuffer);
    DisplayVidBuffer(vidBuffer);
    LoadPalette("palette1");
    SetPalette();

    /* ########## START OF MAIN LOOP ############ */
    k = IsParam(argc, argv, "-lev");

    if(k)
    {
      strcpy(levelName, argv[k+1]);
    } else
    {
      strcpy(levelName, "data\\level1.dat");
    }
    while(!doneGame)
    {
      ret = MainLoop(levelName);

      switch(ret)
      {
        case 0:    /* mission complete */
                   break;
        case 1:    /* player died */
                   ShowPlayerDie();
                   /* Setup variables for the player */
                   me.weapons = 1 + 2 + 4 + 8;
                   me.score = 0;
                   me.curLaser = 0;
                   break;
        case 2:    /* player aborted game */
                   doneGame = 1;
                   break;
      }
      InitLists();
    }
	 TermGraphics();
	 FreeMemory();
    allegro_exit();
  } else
  {
	 printf("\nCan't get memory for video buffer.\n");
	 fflush(stdout);
  }
}


/*
   Main loop of program.
   Returns   0  = Player died
             1  = Mission complete
             2  = Mission aborted
*/
int MainLoop(char *levelName)
{
  register int  i, r;
  int retValue;
  int k;
  int done = 0;
  unsigned long waitTime;
  unsigned long t;
  int slowCnt = 0;
  char buffer[11];
  COORDS t1, t2;
  MBITMAP weapCheck;
  int deadcnt = 0;
  int mouseX = 320, mouseY = 100, mouseB;
  int oldmx = 320, oldmy =  100;
  int usePow;
  int addDistance = 0;
  int curEnemy= 0;
  int tx, ty;
  int ttx, tty;
  char tmpBuf[13];

  ReadLevelData(levelName);
  SetMouseCoords(320, 100);
  pointCount = 0;

   if(useSound)
   {
     play_midi(NULL, FALSE);
     play_midi((MIDI *)item[0], FALSE);
   }

   if(solidBack)
   {
     Melt(bigBack[whichBigBack].bits);
   }

   do {
      t = rawclock();
      waitTime = t + 1;

      if(useSound)
      {
        if(midi_pos == -1)
        {
          if(me.speed < 6)
          {
            play_midi((MIDI *)item[0], FALSE);
          } else
          {
            play_midi((MIDI *)item[1], FALSE);
          }
        }
      }

     /* Do I add another bad guy */
     if(addDistance >= addBadGuyDistance)
     {
       i = 0;
       do {
         if(badGuys[i].action == -1)
         {
           if(curEnemy >= MAX_BADGUYS)
           {
             curEnemy = 0;
           }
           MakeBadGuy(&badGuys[i], badGuyList[curEnemy].type, -50,
                      badGuyList[curEnemy].y);
           curEnemy++;
           addDistance = 0;
           i = NUM_BADONSCREEN;
         }
         i++;
       } while(i < NUM_BADONSCREEN);
     }

     if(solidBack) /* solid full screen background */
     {
       memcpy(vidBuffer, bigBack[whichBigBack].bits, 64000);
       i = 0;
       do {
         *(vidBuffer+i) = 0;
         i++;
       } while(i < 3200);
     } else
     {
       ClearVidBuffer(vidBuffer);
     }

     /* Create a power up if score has gain 5000 points */
     if(pointCount >= 5000)
     {
       pointCount = 0;
       AddPowerUp(random() % NUM_POWER_UPS, -10, 97);
     }

     /* Draw stars */
     if(useStars)
     {
       i = 0;
       do {
         stars[i].x += stars[i].z * me.speed;
         if(stars[i].x > theView.xmax)
         {
           stars[i].x =  theView.xmin;
           stars[i].y=(random() % (theView.ymax-theView.ymin))+theView.ymin;
         }
         if(me.speed < 6)
         {
           switch(stars[i].z)
           {
             case 0:  t = 75; break;
             case 1:  t = 66; break;
             case 2:  t = 57; break;
           }
           PlotPixel(vidBuffer, stars[i].x, stars[i].y, t);
         } else
         {
           switch(stars[i].z)
           {
             case 0:  t = 75; break;
             case 1:  t = 66; break;
             case 2:  t = 57; break;
           }
           DrawLine(stars[i].x, stars[i].y, stars[i].x + 5 * (me.speed - 5),
                    stars[i].y, t);
         }
         i++;
       } while(i < NUM_STARS);
     }

     /* Draw other background objects */
     if(useBackGroundImage)
     {
       i = 0;
       do {
         switch(i)
         {
           case 0: if(me.speed < 6)
                   {
                     if(scaleBack)
                     {
                       TransDrawScaleBitmap(background[i].x, background[i].y,
                                background[i].x+me.disTravel/1000,
                                background[i].y+me.disTravel/1000,
                                160-((background[i].x+me.disTravel/1000)/2),
                                100-((background[i].y+me.disTravel/1000)/2),
                                background[i].bits, 255);
                     } else
                     {
                       TransBitBlit(background[i].bits, vidBuffer,
                            background[i].x, background[i].y,
                            160-(background[i].x/2), 100-(background[i].y/2),
                            255);
                     }
                   }
                   break;
         }
         i++;
       } while(i < NUM_BACKGROUND);
     } 

     /* Draw meteors */
     if(useMeteors)
     {
       i = 0;
       do {
         if(meteors[i].z != -1)
         {
           meteors[i].x += 3 * me.speed;
           if(meteors[i].x > theView.xmax)
           {
             if(meteors[i].remake)
             {
               meteors[i].x = theView.xmin-(random() % 50);
               meteors[i].y = (random()%(theView.ymax-theView.ymin))+
                               theView.ymin;
               meteors[i].z = random() % 2;
             } else
             {
               meteors[i].z = -1;
             }
           }
           if(me.speed < 6)
           {
             if(meteors[i].x < 320)
             {
               TransBitBlit(rocks[meteors[i].z].bits, vidBuffer,
                        rocks[meteors[i].z].x, rocks[meteors[i].z].y,
                        meteors[i].x, meteors[i].y, 255);
             }
           }
           i++;
         }
       } while(i < NUM_METEORS);
     }

     /* Draw enemy ships and do their thinking */
     i = 0;
     do {
       if(badGuys[i].action != -1)  /* is this a bad guy? */
       {
         EnemyThink(i);

         TransBitBlit(enemies[badGuys[i].action+badGuys[i].frame].bits,
              vidBuffer, enemies[badGuys[i].action+badGuys[i].frame].x,
              enemies[badGuys[i].action+badGuys[i].frame].y,
              badGuys[i].x, badGuys[i].y, 255);
       }
       i++;
     } while(i < NUM_BADONSCREEN);

     /* Draw shots */
     i = 0;
     do {
       if(shots[i].type != -1)
       {
         shots[i].x -= 10;
         if(shots[i].x < (theView.xmin-10))
         {
           shots[i].type = -1;
         }
         if(shots[i].type >= 0 && shots[i].type < 4)    /* lasers */
         {
           TransBitBlit(lasers[shots[i].type].bits, vidBuffer,
                        lasers[shots[i].type].x, lasers[shots[i].type].y,
                        shots[i].x, shots[i].y, 255);
         } else if(shots[i].type == 4)
         {
           TransBitBlit(missle[0].bits, vidBuffer, missle[0].x, missle[0].y,
                        shots[i].x, shots[i].y, 255);
         }
       }
       i++;
     } while(i < MAX_SHOTS);

     /* Draw enemy shots */
     i = 0;
     do {
       if(enemyShots[i].type != -1)
       {
         enemyShots[i].x += 10;
         if(enemyShots[i].x > (theView.xmax-1))
         {
           enemyShots[i].type = -1;
         }
         if(enemyShots[i].type >= 0 && enemyShots[i].type < 4) /* lasers */
         {
           TransBitBlit(enLasers[enemyShots[i].type].bits, vidBuffer,
                        enLasers[enemyShots[i].type].x,
                        enLasers[enemyShots[i].type].y,
                        enemyShots[i].x, enemyShots[i].y, 255);
         } else if(enemyShots[i].type == 4)
         {
           /* must change missle sprite */
           TransBitBlit(missle[1].bits, vidBuffer, missle[1].x, missle[1].y,
                        enemyShots[i].x, enemyShots[i].y, 255);
         }
       }
       i++;
     } while(i < MAX_SHOTS);

     /* Draw ship */
     if(me.health >= 0)
     {
       TransBitBlit(ship[me.action+me.frame].bits, vidBuffer,
                    ship[me.action+me.frame].x, ship[me.action + me.frame].y,
                    me.x, me.y, 255);
     }

     /*  Draw explosions */
     i = 0;
     do {
       if(explosions[i].type == 1)
       {
         if(explosions[i].wait == explosions[i].delay)
         {
           TransBitBlit(exp[explosions[i].frame].bits, vidBuffer,
                      exp[explosions[i].frame].x,exp[explosions[i].frame].y,
                      explosions[i].x, explosions[i].y, 255);
           explosions[i].frame++;
           if(explosions[i].frame >= NUM_EXP_FRAMES)
           {
             explosions[i].type = -1;
           }
         } else
         {
           explosions[i].wait++;
         }
       }
       i++;
     } while(i < MAX_EXPLOSIONS);

     /* Draw power ups */
     i = 0;
     do {
       if(powerUpInfo[i].z != -1)
       {
         TransBitBlit(powerUps[powerUpInfo[i].z].bits, vidBuffer,
                      powerUps[powerUpInfo[i].z].x,
                      powerUps[powerUpInfo[i].z].y,
                      powerUpInfo[i].x, powerUpInfo[i].y, 255);
       }
       i++;
     } while(i < MAX_POWER_UPS);

     /* Draw weapon power */
     i = 0;
     do {
       if(me.weaponPower >= ((i+1) * 5))
       {
         DrawLine2(i+1, 1, i+1, 8, 233);
       } else
       {
         DrawLine2(i+1, 1, i+1, 8, 0);
       }
       i++;
     } while(i < 20);

     DrawLine2(0, 0, 21, 0, 57);
     DrawLine2(0, 0, 0, 9, 57);
     DrawLine2(0, 9, 21, 9, 57);
     DrawLine2(21, 0, 21, 9, 57);

     /* Draw drive power */
     i = 0;
     do {
       if(me.drivePower >= ((i+1) * 10))
       {
         DrawLine2(i+31, 1, i+31, 8, 218);
       } else
       {
         DrawLine2(i+31, 1, i+31, 8, 0);
       }
       i++;
     } while(i < 20);

     DrawLine2(30, 0, 51, 0, 57);
     DrawLine2(30, 0, 30, 9, 57);
     DrawLine2(30, 9, 51, 9, 57);
     DrawLine2(51, 0, 51, 9, 57);

     /* Draw Boss power bar FIX IT!!!*/
     if(bossOnScreen)
     {
       i = 0;
       do {
         if(me.disTravel >= ((i+1) * 1000))
         {
           DrawLine2(i+61, 1, i+61, 8, 189);
         } else
         {
           DrawLine2(i+61, 1, i+61, 8, 0);
         }
         i++;
       } while(i < 100);
       DrawLine2(60, 0, 161, 0, 57);
       DrawLine2(60, 0, 60, 9, 57);
       DrawLine2(60, 9, 161, 9, 57);
       DrawLine2(161, 0, 161, 9, 57);
     }

     /* Draw health */
     if(me.health > 0)
     {
       i = 0;
       do {
         BitBlit2(health1.bits, vidBuffer, 8, 8, 170+(i*9), 0);
         i++;
       } while(i < me.health);
     }

     /* Draw score */
     sprintf(buffer, "%.10ld", me.score);
     i = 0;
     do {
       BitBlit2(numbers[buffer[i]-48].bits, vidBuffer, 8, 8, 230 + i*9, 0);
       i++;
     } while(i < 10);

     /* Draw the current weapon icon */
     TransBitBlit(weaponIcon[me.curLaser].bits, vidBuffer,
                  weaponIcon[me.curLaser].x,weaponIcon[me.curLaser].y,
                  319-weaponIcon[me.curLaser].x,
                  199-weaponIcon[me.curLaser].y, 255);

     /* Draw the all the missle icons */
     for(i = 0; i < me.numMissles; i++)
     {
       TransBitBlit(weaponIcon[4].bits, vidBuffer,
                    weaponIcon[4].x, weaponIcon[4].y,
                    279-(i * (weaponIcon[4].x+1)),
                    199-weaponIcon[4].y, 255);
     }

     /* check for collisions */
     /* see if any player shots hit meteors */
     if(useMeteors)
     {
       if(me.speed < 6)
       {
         i = 0;
         do {
           if(shots[i].type != -1)
           {
             r = 0;
             do {
               if(meteors[r].z != -1)
               {
                 t1.xmin = meteors[r].x;
                 t1.xmax = meteors[r].x + rocks[meteors[r].z].x;
                 t1.ymin = meteors[r].y;
                 t1.ymax = meteors[r].y + rocks[meteors[r].z].y;
                 tx = meteors[r].x + (rocks[meteors[r].z].x/2);
                 ty = meteors[r].y + (rocks[meteors[r].z].y/2);

                 if(shots[i].type >= 0 && shots[i].type < 4) /* lasers */
                 {
                   t2.xmin = shots[i].x;
                   t2.xmax = shots[i].x + lasers[shots[i].type].x;
                   t2.ymin = shots[i].y;
                   t2.ymax = shots[i].y + lasers[shots[i].type].y;
                   weapCheck.x = lasers[shots[i].type].x;
                   weapCheck.y = lasers[shots[i].type].y;
                   weapCheck.bits = lasers[shots[i].type].bits;
                 } else if(shots[i].type == 4) /* missle */
                 {
                   t2.xmin = shots[i].x;
                   t2.xmax = shots[i].x + missle[0].x;
                   t2.ymin = shots[i].y;
                   t2.ymax = shots[i].y + missle[0].y;
                   weapCheck.x = missle[0].x;
                   weapCheck.y = missle[0].y;
                   weapCheck.bits = missle[0].bits;
                 }

                 if(t2.xmin >= t1.xmin && t2.ymin >= t1.ymin)
                 {
                   if(t2.xmin >= t1.xmin && t2.xmin < t1.xmax &&
                      t2.ymin >= t1.ymin && t2.ymin < t1.ymax)
                   {
                     if(DidCollide1(rocks[meteors[r].z], weapCheck,
                              t1.xmin, t1.ymin, t2.xmin, t2.ymin))
                     {
                       me.score += 10;
                       pointCount += 10;
                       if(useSound)
                       {
                         play_sample((SAMPLE *)item[4],vol,pan, freq, FALSE);
                       }
                       StandardExplosion(tx, ty, 1, 2);
                       meteors[r].z -= WeaponDamage(shots[i].type);
                       if(meteors[r].z < 0)
                       {
                         if(meteors[r].remake)
                         {
                           meteors[r].x = theView.xmin-(random()  % 50);
                           meteors[r].y = (random()%(theView.ymax-
                                           theView.ymin))+theView.ymin;
                           meteors[r].z = random() % 2;
                         } else
                         {
                           meteors[r].z = -1;
                         }
                       }
                       shots[i].type = -1;
                       /* Stop checking against meteors. */
                       r = NUM_METEORS;
                     }
                   }
                 } else if(t2.xmin < t1.xmin && t2.ymin < t1.ymin)
                 {
                   if(t1.xmin >= t2.xmin && t1.xmin < t2.xmax &&
                      t1.ymin >= t2.ymin && t1.ymin < t2.ymax)
                   {
                     if(DidCollide2(rocks[meteors[r].z], weapCheck,
                              t1.xmin, t1.ymin, t2.xmin, t2.ymin))
                     {
                       me.score += 10;
                       pointCount += 10;
                       if(useSound)
                       {
                         play_sample((SAMPLE *)item[4], vol, pan,freq,FALSE);
                       }
                       StandardExplosion(tx, ty, 1, 2);
                       meteors[r].z -= WeaponDamage(shots[i].type);
                       if(meteors[r].z < 0)
                       {
                         if(meteors[r].remake)
                         {
                           meteors[r].x = theView.xmin-(random()  % 50);
                           meteors[r].y = (random()%(theView.ymax-
                                           theView.ymin))+theView.ymin;
                           meteors[r].z = random() % 2;
                         } else
                         {
                           meteors[r].z = -1;
                         }
                       }
                       shots[i].type = -1;
                       /* Stop checking against meteors. */
                       r = NUM_METEORS;
                     }
                   }
                 } else if(t2.xmin >= t1.xmin && t2.ymin < t1.ymin)
                 {
                   if(t2.xmin >= t1.xmin && t2.xmin < t1.xmax &&
                      t1.ymin >= t2.ymin && t1.ymin < t2.ymax)
                   {
                     if(DidCollide3(rocks[meteors[r].z], weapCheck,
                               t1.xmin, t1.ymin, t2.xmin, t2.ymin))
                     {
                       me.score += 10;
                       pointCount += 10;
                       if(useSound)
                       {
                         play_sample((SAMPLE *)item[4],vol,pan, freq, FALSE);
                       }
                       StandardExplosion(tx, ty, 1, 2);
                       meteors[r].z -= WeaponDamage(shots[i].type);
                       if(meteors[r].z < 0)
                       {
                         if(meteors[r].remake)
                         {
                           meteors[r].x = theView.xmin-(random() % 50);
                           meteors[r].y = (random()%(theView.ymax-
                                          theView.ymin))+theView.ymin;
                           meteors[r].z = random() % 2;
                         } else
                         {
                           meteors[r].z = -1;
                         }
                       }
                       shots[i].type = -1;
                       /* Stop checking against meteors. */
                       r = NUM_METEORS;
                     }
                   }
                 } else if(t2.xmin < t1.xmin && t2.ymin >= t1.ymin)
                 {
                   if(t1.xmin >= t2.xmin && t1.xmin < t2.xmax &&
                      t2.ymin >= t1.ymin && t2.ymin < t1.ymax)
                   {
                     if(DidCollide4(rocks[meteors[r].z], weapCheck,
                                 t1.xmin, t1.ymin, t2.xmin, t2.ymin))
                     {
                       me.score += 10;
                       pointCount += 10;
                       if(useSound)
                       {
                         play_sample((SAMPLE *)item[4],vol,pan,freq, FALSE);
                       }
                       StandardExplosion(tx, ty, 1, 2);
                       meteors[r].z -= WeaponDamage(shots[i].type);
                       if(meteors[r].z < 0)
                       {
                         if(meteors[r].remake)
                         {
                           meteors[r].x = theView.xmin-(random() % 50);
                           meteors[r].y = (random()%(theView.ymax-
                                          theView.ymin))+theView.ymin;
                           meteors[r].z = random() % 2;
                         } else
                         {
                           meteors[r].z = -1;
                         }
                       }
                       shots[i].type = -1;
                       /* Stop checking against meteors. */
                       r = NUM_METEORS;
                     }
                   }
                 }
               }
               r++;
             } while(r < NUM_METEORS);
           }
           i++;
         } while(i < MAX_SHOTS);
       }
     }

     /* See if any player shots hit enemy ships */
     i = 0;
     do {
       if(shots[i].type != -1)
       {
         r = 0;
         do {
           if(badGuys[r].action != -1)
           {
             t1.xmin = badGuys[r].x;
             t1.xmax = badGuys[r].x + enemies[badGuys[r].action].x;
             t1.ymin = badGuys[r].y;
             t1.ymax = badGuys[r].y + enemies[badGuys[r].action].y;
             tx =
              badGuys[r].x+(enemies[badGuys[r].action+badGuys[r].frame].x/2);
             ty =
              badGuys[r].y+(enemies[badGuys[r].action+badGuys[r].frame].y/2);

             if(shots[i].type >= 0 && shots[i].type < 4) /* lasers */
             {
               t2.xmin = shots[i].x;
               t2.xmax = shots[i].x + lasers[shots[i].type].x;
               t2.ymin = shots[i].y;
               t2.ymax = shots[i].y + lasers[shots[i].type].y;
               weapCheck.x = lasers[shots[i].type].x;
               weapCheck.y = lasers[shots[i].type].y;
               weapCheck.bits = lasers[shots[i].type].bits;
             } else if(shots[i].type == 4) /* missle */
             {
               t2.xmin = shots[i].x;
               t2.xmax = shots[i].x + missle[0].x;
               t2.ymin = shots[i].y;
               t2.ymax = shots[i].y + missle[0].y;
               weapCheck.x = missle[0].x;
               weapCheck.y = missle[0].y;
               weapCheck.bits = missle[0].bits;
             } 
             if(t2.xmin >= t1.xmin && t2.ymin >= t1.ymin)
             {
               if(t2.xmin >= t1.xmin && t2.xmin < t1.xmax &&
                  t2.ymin >= t1.ymin && t2.ymin < t1.ymax)
               {
                 if(DidCollide1(enemies[badGuys[r].action], weapCheck,
                                  t1.xmin, t1.ymin, t2.xmin, t2.ymin))
                 {
                   me.score += 10;
                   pointCount += 10;
                   if(useSound)
                   {
                     play_sample((SAMPLE *)item[4], vol, pan, freq, FALSE);
                   }
                   StandardExplosion(tx, ty, 1, 2);
                   badGuys[r].health -= WeaponDamage(shots[i].type);
                   if(badGuys[r].health < 0)
                   {
                     badGuys[r].action = -1;
                     me.score += badGuys[r].score;
                     pointCount += badGuys[r].score;
                   }
                   shots[i].type = -1;
                   r = NUM_BADONSCREEN;
                 }
               }
             } else if(t2.xmin < t1.xmin && t2.ymin < t1.ymin)
             {
               if(t1.xmin >= t2.xmin && t1.xmin < t2.xmax &&
                  t1.ymin >= t2.ymin && t1.ymin < t2.ymax)
               {
                 if(DidCollide2(enemies[badGuys[r].action], weapCheck,
                             t1.xmin, t1.ymin, t2.xmin, t2.ymin))
                 {
                   me.score += 10;
                   pointCount += 10;
                   if(useSound)
                   {
                     play_sample((SAMPLE *)item[4], vol, pan, freq, FALSE);
                   }
                   StandardExplosion(tx, ty, 1, 2);
                   badGuys[r].health -= WeaponDamage(shots[i].type);
                   if(badGuys[r].health < 0)
                   {
                     badGuys[r].action = -1;
                     me.score += badGuys[r].score;
                     pointCount += badGuys[r].score;
                   }
                   shots[i].type = -1;
                   r = NUM_BADONSCREEN;
                 }
               }
             } else if(t2.xmin >= t1.xmin && t2.ymin < t1.ymin)
             {
               if(t2.xmin >= t1.xmin && t2.xmin < t1.xmax &&
                  t1.ymin >= t2.ymin && t1.ymin < t2.ymax)
               {
                 if(DidCollide3(enemies[badGuys[r].action], weapCheck,
                                  t1.xmin, t1.ymin, t2.xmin, t2.ymin))
                 {
                   me.score += 10;
                   pointCount += 10;
                   if(useSound)
                   {
                     play_sample((SAMPLE *)item[4], vol, pan, freq, FALSE);
                   }
                   StandardExplosion(tx, ty, 1, 2);
                   badGuys[r].health -= WeaponDamage(shots[i].type);
                   if(badGuys[r].health < 0)
                   {
                     badGuys[r].action = -1;
                     me.score += badGuys[r].score;
                     pointCount += badGuys[r].score;
                   }
                   shots[i].type = -1;
                   r = NUM_BADONSCREEN;
                 }
               }
             } else if(t2.xmin < t1.xmin && t2.ymin >= t1.ymin)
             {
               if(t1.xmin >= t2.xmin && t1.xmin < t2.xmax &&
                  t2.ymin >= t1.ymin && t2.ymin < t1.ymax)
               {
                 if(DidCollide4(enemies[badGuys[r].action], weapCheck,
                                  t1.xmin, t1.ymin, t2.xmin, t2.ymin))
                 {
                   me.score += 10;
                   pointCount += 10;
                   if(useSound)
                   {
                     play_sample((SAMPLE *)item[4], vol, pan, freq, FALSE);
                   }
                   StandardExplosion(tx, ty, 1, 2);
                   badGuys[r].health -= WeaponDamage(shots[i].type);
                   if(badGuys[r].health < 0)
                   {
                     badGuys[r].action = -1;
                     me.score += badGuys[r].score;
                     pointCount += badGuys[r].score;
                   }
                   shots[i].type = -1;
                   r = NUM_BADONSCREEN;
                 }
               }
             }
           }
           r++;
         } while(r < NUM_BADONSCREEN);
       }
       i++;
     } while(i < MAX_SHOTS);


     /* See if meteors hit ship */
     if(useMeteors)
     {
       if(me.health  >= 0)
       {
         if(me.speed < 6)
         {
           i = 0;
           do {
             if(meteors[i].z != -1)
             {
               t1.xmin = meteors[i].x;
               t1.xmax = meteors[i].x + (rocks[meteors[i].z].x);
               t1.ymin = meteors[i].y;
               t1.ymax = meteors[i].y + (rocks[meteors[i].z].y);

               t2.xmin = me.x;
               t2.xmax = me.x + (ship[me.action + me.frame].x);
               t2.ymin = me.y;
               t2.ymax = me.y + (ship[me.action + me.frame].y);
               tx = me.x + (ship[me.action + me.frame].x/2);
               ty = me.y + (ship[me.action + me.frame].y/2);

               if(t2.xmin > t1.xmin && t2.ymin > t1.ymin)
               {
                 if(t2.xmin >= t1.xmin && t2.xmin < t1.xmax &&
                    t2.ymin >= t1.ymin && t2.ymin < t1.ymax)
                 {
                   if(DidCollide1(rocks[meteors[i].z],
                      ship[me.action+me.frame],
                      t1.xmin, t1.ymin, t2.xmin, t2.ymin))
                   {
                     me.score += 10;
                     pointCount += 10;
                     if(!godMode)
                     {
                       me.health--;
                     }
                     StandardExplosion(tx, ty, 1, 2);
                     if(useSound)
                     {
                       play_sample((SAMPLE *)item[4], vol, pan, freq, FALSE);
                     }
                     if(meteors[i].remake)
                     {
                       meteors[i].x = theView.xmin-(random() % 50);
                       meteors[i].y = (random()%(theView.ymax-theView.ymin))+
                                      theView.ymin;
                       meteors[i].z = random() % 2;
                     } else
                     {
                       meteors[i].z = -1;
                     }
                   }
                 }
               } else if(t2.xmin < t1.xmin && t2.ymin < t1.ymin)
               {
                 if(t1.xmin >= t2.xmin && t1.xmin < t2.xmax &&
                    t1.ymin >= t2.ymin && t1.ymin < t2.ymax)
                 { 
                   if(DidCollide2(rocks[meteors[i].z],
                          ship[me.action+me.frame],
                          t1.xmin, t1.ymin, t2.xmin, t2.ymin))
                   {
                     me.score += 10;
                     pointCount += 10;
                     if(!godMode)
                     {
                       me.health--;
                     }
                     StandardExplosion(tx, ty, 1, 2);
                     if(useSound)
                     {
                       play_sample((SAMPLE *)item[4], vol, pan, freq, FALSE);
                     }
                     if(meteors[i].remake)
                     {
                       meteors[i].x = theView.xmin-(random() % 50);
                       meteors[i].y = (random()%(theView.ymax-theView.ymin))+
                                      theView.ymin;
                       meteors[i].z = random() % 2;
                     } else
                     {
                       meteors[i].z = -1;
                     }
                   }
                 }
               } else if(t2.xmin > t1.xmin && t2.ymin < t1.ymin)
               {
                 if(t2.xmin >= t1.xmin && t2.xmin < t1.xmax &&
                    t1.ymin >= t2.ymin && t1.ymin < t2.ymax)
                 {
                   if(DidCollide3(rocks[meteors[i].z], ship[me.action+me.frame],
                             t1.xmin, t1.ymin, t2.xmin, t2.ymin))
                   {
                     me.score += 10;
                     pointCount += 10;
                     if(!godMode)
                     {
                       me.health--;
                     }
                     StandardExplosion(tx, ty, 1, 2);
                     if(useSound)
                     {
                       play_sample((SAMPLE *)item[4], vol, pan, freq, FALSE);
                     }
                     if(meteors[i].remake)
                     {
                       meteors[i].x = theView.xmin-(random() % 50);
                       meteors[i].y = (random()%(theView.ymax-theView.ymin))+
                                      theView.ymin;
                       meteors[i].z = random() % 2;
                     } else
                     {
                       meteors[i].z = -1;
                     }
                   }
                 }
               } else if(t2.xmin < t1.xmin && t2.ymin > t1.ymin)
               {
                 if(t1.xmin >= t2.xmin && t1.xmin < t2.xmax &&
                    t2.ymin >= t1.ymin && t2.ymin < t1.ymax)
                 {
                   if(DidCollide4(rocks[meteors[i].z], ship[me.action+me.frame],
                                 t1.xmin, t1.ymin, t2.xmin, t2.ymin))
                   {
                     me.score += 10;
                     pointCount += 10;
                     if(!godMode)
                     {
                       me.health--;
                     }
                     StandardExplosion(tx, ty, 1, 2);
                     if(useSound)
                     {
                       play_sample((SAMPLE *)item[4], vol, pan, freq, FALSE);
                     }
                     if(meteors[i].remake)
                     {
                       meteors[i].x = theView.xmin-(random() % 50);
                       meteors[i].y = (random()%(theView.ymax-theView.ymin))+
                                      theView.ymin;
                       meteors[i].z = random() % 2;
                     } else
                     {
                       meteors[i].z = -1;
                     }
                   }
                 }
               }
             }
             i++;
           } while(i < NUM_METEORS);
         }
       }
     } 

     /* See if ship hit any enemy shots */
     if(me.health >= 0)
     {
       i = 0;
       do {
         if(meteors[i].z != -1)
         {
           t1.xmin = enemyShots[i].x;
           t1.xmax = enemyShots[i].x + (enLasers[enemyShots[i].type].x);
           t1.ymin = enemyShots[i].y;
           t1.ymax = enemyShots[i].y + (enLasers[enemyShots[i].type].y);

           t2.xmin = me.x;
           t2.xmax = me.x + (ship[me.action + me.frame].x);
           t2.ymin = me.y;
           t2.ymax = me.y + (ship[me.action + me.frame].y);
           tx = me.x + (ship[me.action + me.frame].x/2);
           ty = me.y + (ship[me.action + me.frame].y/2);

           if(t2.xmin > t1.xmin && t2.ymin > t1.ymin)
           {
             if(t2.xmin >= t1.xmin && t2.xmin < t1.xmax &&
                t2.ymin >= t1.ymin && t2.ymin < t1.ymax)
             {
               if(DidCollide1(enLasers[enemyShots[i].type],
                  ship[me.action+me.frame],
                  t1.xmin, t1.ymin, t2.xmin, t2.ymin))
               {
                 if(!godMode)
                 {
                   me.health--;
                 }
                 StandardExplosion(tx, ty, 1, 2);
                 if(useSound)
                 {
                   play_sample((SAMPLE *)item[4], vol, pan, freq, FALSE);
                 }
                 enemyShots[i].type = -1;
               }
             }
           } else if(t2.xmin < t1.xmin && t2.ymin < t1.ymin)
           {
             if(t1.xmin >= t2.xmin && t1.xmin < t2.xmax &&
                t1.ymin >= t2.ymin && t1.ymin < t2.ymax)
             {
               if(DidCollide2(enLasers[enemyShots[i].type],
                    ship[me.action+me.frame],
                    t1.xmin, t1.ymin, t2.xmin, t2.ymin))
               {
                 if(!godMode)
                 {
                   me.health--;
                 }
                 StandardExplosion(tx, ty, 1, 2);
                 if(useSound)
                 {
                   play_sample((SAMPLE *)item[4], vol, pan, freq, FALSE);
                 }
                 enemyShots[i].type = -1;
               }
             }
           } else if(t2.xmin > t1.xmin && t2.ymin < t1.ymin)
           {
             if(t2.xmin >= t1.xmin && t2.xmin < t1.xmax &&
                t1.ymin >= t2.ymin && t1.ymin < t2.ymax)
             {
               if(DidCollide3(enLasers[enemyShots[i].type],
                    ship[me.action+me.frame],
                    t1.xmin, t1.ymin, t2.xmin, t2.ymin))
               {
                 if(!godMode)
                 {
                   me.health--;
                 }
                 StandardExplosion(tx, ty, 1, 2);
                 if(useSound)
                 {
                   play_sample((SAMPLE *)item[4], vol, pan, freq, FALSE);
                 }
                 enemyShots[i].type = -1;
               }
             }
           } else if(t2.xmin < t1.xmin && t2.ymin > t1.ymin)
           {
             if(t1.xmin >= t2.xmin && t1.xmin < t2.xmax &&
                t2.ymin >= t1.ymin && t2.ymin < t1.ymax)
             {
               if(DidCollide4(enLasers[enemyShots[i].type],
                     ship[me.action+me.frame],
                     t1.xmin, t1.ymin, t2.xmin, t2.ymin))
               {
                 if(!godMode)
                 {
                   me.health--;
                 }
                 StandardExplosion(tx, ty, 1, 2);
                 if(useSound)
                 {
                   play_sample((SAMPLE *)item[4], vol, pan, freq, FALSE);
                 }
                 enemyShots[i].type = -1;
               }
             }
           }
         }
         i++;
       } while(i < MAX_SHOTS);
     }

     /* See if play ship hits any enemy ships */
     if(me.health >= 0)
     {
       i = 0;
       do {
         if(badGuys[i].action != -1)
         {
           t1.xmin = badGuys[i].x;
           t1.xmax = badGuys[i].x + (enemies[badGuys[i].action].x);
           t1.ymin = badGuys[i].y;
           t1.ymax = badGuys[i].y + (enemies[badGuys[i].action].y);
           tx =
             badGuys[i].x+(enemies[badGuys[i].action+badGuys[i].frame].x/2);
           ty =
             badGuys[i].y+(enemies[badGuys[i].action+badGuys[i].frame].y/2);
           t2.xmin = me.x;
           t2.xmax = me.x + (ship[me.action + me.frame].x);
           t2.ymin = me.y;
           t2.ymax = me.y + (ship[me.action + me.frame].y);
           ttx = me.x + (ship[me.action + me.frame].x / 2);
           tty = me.y + (ship[me.action + me.frame].y / 2);

           if(t2.xmin > t1.xmin && t2.ymin > t1.ymin)
           {
             if(t2.xmin >= t1.xmin && t2.xmin < t1.xmax &&
                t2.ymin >= t1.ymin && t2.ymin < t1.ymax)
             {
               if(DidCollide1(enemies[badGuys[i].action],
                     ship[me.action+me.frame],
                     t1.xmin, t1.ymin, t2.xmin, t2.ymin))
               {
                 if(!godMode)
                 {
                   me.health--;
                 }
                 badGuys[i].health--;
                 if(badGuys[i].health < 0)
                 {
                   me.score += badGuys[i].score;
                   pointCount += badGuys[i].score;
                   badGuys[i].action = -1;
                 }
                 StandardExplosion(tx, ty, 1, 2);
                 StandardExplosion(ttx, tty, 1, 2);
                 if(useSound)
                 {
                   play_sample((SAMPLE *)item[4], vol, pan, freq, FALSE);
                 }
               }
             }
           } else if(t2.xmin < t1.xmin && t2.ymin < t1.ymin)
           {
             if(t1.xmin >= t2.xmin && t1.xmin < t2.xmax &&
                t1.ymin >= t2.ymin && t1.ymin < t2.ymax)
             {
               if(DidCollide2(enemies[badGuys[i].action],
                      ship[me.action+me.frame],
                      t1.xmin, t1.ymin, t2.xmin, t2.ymin))
               {
                 if(!godMode)
                 {
                   me.health--;
                 }
                 badGuys[i].health--;
                 if(badGuys[i].health < 0)
                 {
                   me.score += badGuys[i].score;
                   pointCount += badGuys[i].score;
                   badGuys[i].action = -1;
                 }
                 StandardExplosion(tx, ty, 1, 2);
                 StandardExplosion(ttx, tty, 1, 2);
					  if(useSound)
					  {
					    play_sample((SAMPLE *)item[4], vol, pan, freq, FALSE);
				     }
               }
             }
           } else if(t2.xmin > t1.xmin && t2.ymin < t1.ymin)
		  	  {
			    if(t2.xmin >= t1.xmin && t2.xmin < t1.xmax &&
				    t1.ymin >= t2.ymin && t1.ymin < t2.ymax)
			    {
			      if(DidCollide3(enLasers[enemyShots[i].type],
                        ship[me.action+me.frame],
								t1.xmin,	t1.ymin,	t2.xmin,	t2.ymin))
				   {
					  if(!godMode)
					  {
					    me.health--;
					  }
                 badGuys[i].health--;
                 if(badGuys[i].health < 0)
                 {
                   me.score += badGuys[i].score;
                   pointCount += badGuys[i].score;
                   badGuys[i].action = -1;
                 }
                 StandardExplosion(tx, ty, 1, 2);
                 StandardExplosion(ttx, tty, 1, 2);
					  if(useSound)
					  {
					    play_sample((SAMPLE *)item[4], vol, pan, freq, FALSE);
					  }
               }
             }
			  } else if(t2.xmin < t1.xmin && t2.ymin > t1.ymin)
			  {
			    if(t1.xmin >= t2.xmin && t1.xmin < t2.xmax &&
			       t2.ymin >= t1.ymin && t2.ymin < t1.ymax)
			    {
			      if(DidCollide4(enemies[badGuys[i].action],
                        ship[me.action+me.frame],
								t1.xmin, t1.ymin, t2.xmin, t2.ymin))
               {
					  if(!godMode)
				  	  {
					    me.health--;
					  }
                 badGuys[i].health--;
                 if(badGuys[i].health < 0)
                 {
                   me.score += badGuys[i].score;
                   pointCount += badGuys[i].score;
                   badGuys[i].action = -1;
                 }
                 StandardExplosion(tx, ty, 1, 2);
                 StandardExplosion(ttx, tty, 1, 2);
					  if(useSound)
					  {
					    play_sample((SAMPLE *)item[4], vol, pan, freq, FALSE);
					  }
               }
             }
           }
         }
		   i++;
       } while(i < NUM_BADONSCREEN);
	  }

	  /* See if ship hit any power ups */
	  if(me.health	>=	0)
	  {
		 i	= 0;
		 do {
			if(powerUpInfo[i].z != -1)
			{
			  t1.xmin =	powerUpInfo[i].x;
			  t1.xmax =	powerUpInfo[i].x + (powerUps[powerUpInfo[i].z].x);
			  t1.ymin =	powerUpInfo[i].y;
			  t1.ymax =	powerUpInfo[i].y + (powerUps[powerUpInfo[i].z].y);
			  t2.xmin =	me.x;
			  t2.xmax =	me.x + (ship[me.action + me.frame].x);
			  t2.ymin =	me.y;
			  t2.ymax =	me.y + (ship[me.action + me.frame].y);

			  if(t2.xmin >	t1.xmin && t2.ymin >	t1.ymin)
			  {
				 if(t2.xmin	>=	t1.xmin && t2.xmin <	t1.xmax &&
					 t2.ymin	>=	t1.ymin && t2.ymin <	t1.ymax)
				 {
					if(DidCollide1(powerUps[powerUpInfo[i].z],
	                           ship[me.action+me.frame],
										t1.xmin,	t1.ymin,	t2.xmin,	t2.ymin))
					{
					  UsePowerUp(powerUpInfo[i].z);
                 powerUpInfo[i].z = -1;
					  if(useSound)
					  {
                   play_sample((SAMPLE *)item[6], vol, pan, freq, FALSE);
					  }
					}
				 }
			  } else	if(t2.xmin < t1.xmin	&&	t2.ymin < t1.ymin)
			  {
				 if(t1.xmin	>=	t2.xmin && t1.xmin <	t2.xmax &&
					 t1.ymin	>=	t2.ymin && t1.ymin <	t2.ymax)
				 {
					if(DidCollide2(powerUps[powerUpInfo[i].z],
	                           ship[me.action+me.frame],
										t1.xmin,	t1.ymin,	t2.xmin,	t2.ymin))
					{
                 UsePowerUp(powerUpInfo[i].z);
                 powerUpInfo[i].z = -1;
					  if(useSound)
					  {
                   play_sample((SAMPLE *)item[6], vol, pan, freq, FALSE);
					  }
					}
				 }
			  } else	if(t2.xmin > t1.xmin	&&	t2.ymin < t1.ymin)
			  {
				 if(t2.xmin	>=	t1.xmin && t2.xmin <	t1.xmax &&
					 t1.ymin	>=	t2.ymin && t1.ymin <	t2.ymax)
				 {
					if(DidCollide3(powerUps[powerUpInfo[i].z],
	                           ship[me.action+me.frame],
										t1.xmin,	t1.ymin,	t2.xmin,	t2.ymin))
					{
                 UsePowerUp(powerUpInfo[i].z);
                 powerUpInfo[i].z = -1;
					  if(useSound)
					  {
                   play_sample((SAMPLE *)item[6], vol, pan, freq, FALSE);
					  }
					}
				 }
			  } else	if(t2.xmin < t1.xmin	&&	t2.ymin > t1.ymin)
			  {
				 if(t1.xmin	>=	t2.xmin && t1.xmin <	t2.xmax &&
					 t2.ymin	>=	t1.ymin && t2.ymin <	t1.ymax)
				 {
					if(DidCollide4(powerUps[powerUpInfo[i].z],
	                           ship[me.action+me.frame],
										t1.xmin,	t1.ymin,	t2.xmin,	t2.ymin))
					{
                 UsePowerUp(powerUpInfo[i].z);
                 powerUpInfo[i].z = -1;
					  if(useSound)
					  {
                   play_sample((SAMPLE *)item[6], vol, pan, freq, FALSE);
					  }
					}
				 }
			  }
			}
			i++;
		 }	while(i < MAX_POWER_UPS);
	  }


	  /* Display image */
	  WaitRetrace();
	  DisplayVidBuffer(vidBuffer);

	  /* Recalculate power up positions */
	  i =	0;
	  do {
        if(powerUpInfo[i].z != -1)
		  {
		 	 powerUpInfo[i].x += 3;
			 if(powerUpInfo[i].x > 319)
			 {
            powerUpInfo[i].z = -1;
			 }
		  }
		  i++;
	  } while(i < MAX_POWER_UPS);

     /* Rotate ship frames */
	  me.frame++;
	  switch(me.action)
	  {
		 case	MOVE:	 if(me.frame == (MOVE_FRAMES - 1))
                   {
                     me.frame = 0;
                   }
                   break;
	  }

	  /* increase distance traveled */
	  me.disTravel += me.speed;
     addDistance += me.speed;

	  if(me.disTravel	> 100000)
	  {
       me.disTravel = 0;
	  }

	  /* increase weapon power */
	  me.weaponPower++;
	  if(me.weaponPower > me.maxWeaponPower)
	  {
		 me.weaponPower =	me.maxWeaponPower;
	  }

	  /* increase drive power if weapon power 100% */
	  if(me.weaponPower == me.maxWeaponPower && me.speed < 6)
	  {
		 me.drivePower++;
		 if(me.drivePower	> me.maxDrivePower)
		 {
			me.drivePower = me.maxDrivePower;
		 }
	  }
	  /* if out of drive power at warp slow down. */
	  if(me.drivePower == 0	&&	me.speed	> 5)
	  {
		 slowCnt++;
	  }
	  if(me.drivePower == 0	&&	me.speed	> 5 && slowCnt	> 10)
	  {
		 srandom(45);
		 slowCnt	= 0;
		 me.speed--;
		 if(me.speed == 6)
		 {
			if(useSound)
			{
			  play_midi(NULL,	FALSE);
			  play_midi((MIDI	*)item[0], FALSE);
			}
		 }
	  }
	  /* decrease drive power if at warp */
	  if(me.speed > 5)
	  {
		 me.drivePower--;
		 if(me.drivePower	< 0)
		 {
			me.drivePower = 0;
		 }
	  }

	  /* Get all inputs if still alive */
	  if(me.health	>=	0)
	  {
		 GetMouseCoords(&mouseX, &mouseY, &mouseB);

		 if(mouseX != oldmx)
		 {
			if(mouseX <	oldmx)
			{
			  me.x -= 5;
			  if(me.x <	theView.xmin) me.x =	theView.xmin;
			} else if(mouseX > oldmx)
			{
			  me.x += 5;
			  if(me.x >	(theView.xmax-30)) me.x	= (theView.xmax-30);
			}
		 }
		 if(mouseY != oldmy)
		 {
			if(mouseY <	oldmy)
			{
			  me.y -= 5;
			  if(me.y <	theView.ymin) me.y =	theView.ymin;
			} else if(mouseY > oldmy)
			{
			  me.y += 5;
			  if(me.y >	(theView.ymax-20)) me.y	= (theView.ymax-20);
			}
		 }
		 SetMouseCoords(320,	100);

		 if(mouseB & 1)  /* fire laser */
		 {
			switch(me.curLaser)
			{
			  case 0: usePow = 10; break;
			  case 1: usePow = 15; break;
			  case 2: usePow = 5; break;
			  case 3: usePow = 20; break;
			}
			if(me.weaponPower	>=	usePow)
			{
			  i =	0;
			  do
			  {
				 if(shots[i].type	==	-1)
				 {
					shots[i].x = me.x;
					shots[i].y = me.y	+ 10;
					shots[i].type = me.curLaser;
					me.weaponPower	-=	usePow;
					if(useSound)
					{
					  play_sample((SAMPLE *)item[2],	vol, pan, freq, FALSE);
					}
					break;
				 }
				 i++;
			  } while(i	< MAX_SHOTS);
			}
		 }
		 if(mouseB & 2) /* fire missle */
		 {
			if(me.numMissles > 0)
			{
			  i =	0;
			  do {
				 if(shots[i].type	==	-1)
				 {
					me.numMissles--;
					shots[i].x = me.x;
					shots[i].y = me.y	+ 10;
					shots[i].type = 4;
					if(useSound)
					{
					  play_sample((SAMPLE *)item[5],	vol, pan, freq, FALSE);
					}
					i = MAX_SHOTS + 1;
				 }
				 i++;
			  } while(i	< MAX_SHOTS);
			}
		 }

		 if(keypressed())
		 {
         if(key[KEY_UP]) /* Up arrow */
			{
			  me.y -= 5;
           if(me.y < theView.ymin) me.y = theView.ymin;
			}
			if(key[KEY_DOWN])	/* down arrow */
			{
			  me.y += 5;
			  if(me.y > (theView.ymax - 20))
			  {
			    me.y =  theView.ymax - 20;
           }
			}
         if(key[KEY_LEFT])	/* left arrow */
			{
			  me.x -= 5;
			  if(me.x < theView.xmin) me.x = theView.xmin;
			}
			if(key[KEY_RIGHT]) /* right arrow */
			{
			  me.x += 5;
			  if(me.x > (theView.xmax-30))
			  {
			    me.x = (theView.xmax - 30);
           }
			}
    	   if(key[KEY_ESC]) /* ESC key */
         {
			  done = 1;
           retValue = 2;
			}
         if(key[KEY_EQUALS] || key[KEY_PLUS_PAD])	/* speed up */
			{
			  me.speed++;
			  if(me.speed > 5 && !canWarp)
			  {
			 	 me.speed = 5;
			  }
			  if(me.drivePower > 0)
			  {
				 if(me.speed > 8)
             {
               me.speed = 8;
             }
				 if(useSound)
				 {
				   if(me.speed == 6)
				   {
				  	  play_midi(NULL, FALSE);
					  play_midi((MIDI *)item[1], FALSE);
					}
				 }
           } else
			  {
				 if(me.speed > 5)
				 {
					me.speed = 5;
             }
           }
         }
			if(key[KEY_MINUS_PAD] || key[KEY_MINUS])	 /* slow down */
			{
			  me.speed--;
			  if(me.speed < 1)
			  {
			    me.speed = 1;
           }
			  if(useSound)
			  {
			 	 if(me.speed == 5)
				 {
				   play_midi(NULL, FALSE);
				   play_midi((MIDI *)item[0], FALSE);
				 }
			  }
         }
			if(key[KEY_SPACE]) /* fire laser */
			{
			  switch(me.curLaser)
			  {
				 case 0: usePow = 10; break;
				 case 1: usePow = 15; break;
				 case 2: usePow = 5; break;
				 case 3: usePow = 20; break;
           }
			  if(me.weaponPower >= usePow)
			  {
				 i = 0;
				 do
				 {
				   if(shots[i].type == -1)
				   {
				 	  shots[i].x = me.x;
					  shots[i].y = me.y + 10;
					  shots[i].type = me.curLaser;
					  me.weaponPower -= usePow;
					  if(useSound)
					  {
						 play_sample((SAMPLE *)item[2], vol, pan, freq, FALSE);
					  }
					  i = MAX_SHOTS + 1;
					}
					i++;
				 } while(i < MAX_SHOTS);
			  }
         }
			if(key[KEY_ENTER]) /* fire missle */
			{
			  if(me.numMissles > 0)
			  {
				 i = 0;
				 do {
					if(shots[i].type == -1)
					{
					  me.numMissles--;
					  shots[i].x = me.x;
					  shots[i].y = me.y + 10;
					  shots[i].type = 4;
					  if(useSound)
					  {
						 play_sample((SAMPLE *)item[5], vol, pan, freq, FALSE);
					  }
					  i = MAX_SHOTS + 1;
					}
					i++;
				 } while(i < MAX_SHOTS);
			  }
			}
			if(key[KEY_OPENBRACE])	/* previous gun */
			{
			  do {
				 me.curLaser--;
				 if(me.curLaser < 0)
             {
               me.curLaser = NUM_LASERS-1;
             }
			  } while(!HaveWeapon(me.curLaser, me.weapons));
		   }
			if(key[KEY_CLOSEBRACE])	/* next gun */
			{
			  do {
				 me.curLaser++;
				 if(me.curLaser >= NUM_LASERS)
             {
               me.curLaser	= 0;
             }
			  } while(!HaveWeapon(me.curLaser, me.weapons));
			}
         if(key[KEY_F10]) /* screen shot */
         {
            sprintf(tmpBuf, "space000.pcx");
            for(i = 1; i < 1000 && !FileExist(tmpBuf); i++)
            {
              sprintf(&tmpBuf[5], "%03d.pcx", i);
            }
            SavePCXFile(tmpBuf);
         }
		 }
	  } else
	  {
  	    done = 1;
	  }

	  if(!done)
	  {
	    while(waitTime > rawclock()) { }
 	  }
   } while(!done);

   if(me.health < 0)
   {
     retValue = 0;
   }

   clear_keybuf();
   return retValue;
}

void LoadSounds(void)
{
  if(useSound == 1)
  {
    /* load music */
    item[0] = load_midi(soundFiles[0]);
    if(!item[0])
    {
      printf("\nError loading music file \'%s\'.\n", soundFiles[0]);
      FreeMemory();
      allegro_exit();
      exit(1);
    }
    item[1] = load_midi(soundFiles[1]);
    if(!item[1])
    {
      printf("\nError loading music file \'%s\'\n", soundFiles[1]);
      FreeMemory();
      allegro_exit();
      exit(1);
    }
    item[2] = load_sample(soundFiles[2]);
    if(!item[2])
    {
      printf("\nError loading sound file \'%s\'.\n", soundFiles[2]);
      FreeMemory();
      allegro_exit();
      exit(1);
    }
    item[3] = load_midi(soundFiles[3]);
    if(!item[3])
    {
      printf("\nError loading sound file \'%s\'.\n", soundFiles[3]);
      FreeMemory();
      allegro_exit();
      exit(1);
    }
    item[4] = load_sample(soundFiles[4]);
    if(!item[4])
    {
      printf("\nError loading sound file \'%s\'.\n", soundFiles[4]);
      FreeMemory();
      allegro_exit();
      exit(1);
    }
    item[5] = load_sample(soundFiles[5]);
    if(!item[5])
    {
      printf("\nError loading sound file \'%s\'.\n", soundFiles[5]);
      FreeMemory();
      allegro_exit();
      exit(1);
    }
    item[6] = load_sample(soundFiles[6]);
    if(!item[6])
    {
      printf("\nError loading sound file \'%s\'.\n", soundFiles[6]);
      FreeMemory();
      allegro_exit();
      exit(1);
    }
    item[7] = load_sample(soundFiles[7]);
    if(!item[7])
    {
      printf("\nError loading sound file \'%s\'.\n", soundFiles[7]);
      FreeMemory();
      allegro_exit();
      exit(1);
    }
  }
}
